%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve a tracking model with an MA(q) state process using different solution methods 
% (i) AY (2019) method, (ii) Our FOCs based method, (iii) MVFI method based
% on a sequence of static RI problems, (iv) MVFI method using the envelope
% condition, (v) golden rule solution with betat = 1.
%
% Tracking model where the target is a univariate MA(2) process
% y_t = epsilon_t+theta_1*epsilon_(t-1)+theta_2*epsilon_(t-2)
%
% You need to install the software CVX that can be freely 
% downloaded from the website: http://cvxr.com/cvx/
%
%
%% @ Written by Jianjun Miao, Boston University, Email: miaoj@bu.edu 
%           and Jieran Wu, Zhejiang University, Email: jw5ya@zju.edu.cn
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all

cvx_solver mosek

%Generate parameter data, tracking a univariate variable, which has MA(2)
%representation,  y_t = epsilon_t+theta_1*epsilon_(t-1)+theta_2*epsilon_(t-2)

beta = 0.8;    % Discount factor
theta_1 = 0.8;
theta_2 = 0.5;
sigma = 0.5;

L = [sigma, sigma,0]'; 
W = L*L'; 

A =[0, theta_1,theta_2
    0, 0, 0
    0, 1, 0];

G = [1,0,0];

Omega = G'*G; %Weighting matrix
lambda = 0.5;%2;  % Information cost. My lambda corresponds to 2*lambda in Sims handbook example
n = size(A,1); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Solve for the steady state
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sigma_pr0 = A*A'+ W;
Theta0 = Omega;

% Miao's method based on FOC
tstart = tic;
[SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0);
tendSS = toc(tic);

[C,V]=RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda)
B = zeros(n,1); %Exogenous state
F = -G; % policy function y = -F*x = G*x
T = 20;

[X, y,u_full] = RI_IRF2(A,B,C,F,SigmaSS,V,L,T);
t=1:T;
figure
plot(t,y(1,1:T,1)/sigma,'--',t,u_full(1,1:T,1)/sigma);
hold on

%Check second order condition
disp('Check second-order condition: eigenvalues of H2 >=0')
H2 = inv(SigmaSS)-beta*A'*inv(A*SigmaSS*A'+W)*A;
eig(H2)

%Transition dynamics using FOCs
tic
T = 20;
Sigma0 = SigmaSS;
ThetaT = ThetaSS;
Sigma_pr0 = 0.1*(A*A'+W); %Initial prior must be positive semidefinite
[Sigmat, Sigma_prt, Thetat, Lambdat] = RI_TD_FOC(lambda,beta,Omega, A, W, Sigma0, ThetaT,Sigma_pr0,T);
Sigmat;
toc

    for t=1:T-1;
        [C,V,dim(t)] = RI_SIG(Sigmat(:,:,t),Sigma_prt(:,:,t),Thetat(:,:,t),lambda);
        if dim(t)==0
            disp(['t = ',num2str(t-1)]);
        elseif dim(t) ==1
            disp(['One dimension, ' ' t = ',num2str(t-1)])
            C1(:,:,t) = C;
            V1(:,:,t)=V;
        end
    end
    

%AY (2019) method based on FOC
tstart = tic;
[Sigma_pr,SigmaAY,Lambda,Omega0,Y,Sigma_z,K] = Solve_RI_Dynamics(0.5*lambda,beta,A,L,G',Theta0,Sigma_pr0);
tendAY = toc(tstart);

%Golden rule solution, the same as that for beta=1
tstart = tic;
[SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,beta);
tendGR = toc(tstart);


[C,V] = RI_SIG(SigmaGR,Sigma_prGR,ThetaGR,lambda);
B = zeros(n,1); %Exogenous state
F = -G; % policy function y = -F*x = G*x
T = 20;
[X, y,u_full] = RI_IRF2(A,B,C,F,SigmaGR,V,L,T);
t=1:T;

plot(t,y(1,1:T,1)/sigma,'-.');
title('IRF')
legend('RI','Full Infor','Golden Rule')

%MVFI based on a sequence of static RI problems
tstart = tic;
Theta0 = ThetaGR;
Sigma_pr0 = Sigma_prGR; %A*A'+W;
[SigmaSQ,Sigma_pr1,Lambda] = RI_SS_SQ(A,W,Omega,lambda, beta,Sigma_pr0,Theta0);
tendSQ = toc(tstart); 

%Use golden solution as initial guess for Steady state
% MVI using envelope condition and CCP method
tstart = tic;
tol = 1e-6;
a = 0.5;
[SigmaEC,Sigma_pr, Lambda1] = RI_SS_EC(A,W,Omega,lambda,beta, SigmaGR, LambdaGR,tol,a);
tendEC = toc(tstart) 


disp('Check Solutions for different approaches')

disp(['Miao FOC method, Running time: ' num2str(tendSS)])
SigmaSS

disp(['Afrouzi and Young FOC method, Running time: ' num2str(tendAY)])
SigmaAY

disp(['Miao MVFI based on sequence of static models, Running time: ' num2str(tendSQ)])
SigmaSQ

disp(['MFV using envelope condition, Running time: ' num2str(tendEC)])
SigmaEC

disp(['Golden-rule solution, Running time: ' num2str(tendGR)])
SigmaGR









