%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab program to solve the firm investment problem in Section 5.3 of Miao, Wu, and Young (2020)
% Solve for the steady state and transition dynamics of the optimal information structure 
% with discounted information costs.
% Plot impulse response functions
% You need to install the software CVX that can be freely 
% downloaded from the website:
% http://cvxr.com/cvx/
%
% @ Written by Jianjun Miao, Boston University, Email: miaoj@bu.edu 
%           and Jieran Wu, Zhejiang University, Email: jw5ya@zju.edu.cn
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Model parameter values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ns = 2;  %number of endogenous states (k1,k2)
nz = 3;  %number of exogenous states (including the constant state) (1,z,e)
nd = 2;  %number of controls (I1,I2)
nst = nz+ns; %total number of states

%parameters
alpha = 0.26; theta = 0.076; beta = 0.9615; delta1=0.126; delta2=0.05;
rhoz = 0.91; rhoe = 0.0; tau=0.35; sigmak1 = 0.01; sigmak2 = 0.01;
sigmaz = 0.01; sigmae = 0.1; chi = 0.5;

algo = 2;  % If algo =1, use the FOCs based methods; 
           % If algo =2, use the MVI method based on a sequence of static RI problems
           % If algo =3, use the MVF method using the envelope condition
           
cvx_solver mosek %Use sedumi or sdpt3 if mosek reports errors  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
%Nonstochastic steady state
k1bar = (alpha*beta*(1-tau)/(1-beta*(1-delta1))*(theta*beta*(1-tau)/(1-beta*(1-delta2))/(1-chi*tau))^(theta/(1-theta)))^((1-theta)/(1-theta-alpha));
k2bar = (theta*beta*(1-tau)/(1-beta*(1-delta2))/(1-chi*tau))^(1/(1-theta))*k1bar^(alpha/(1-theta));
SS = [0.0;0.0; k1bar; k2bar; delta1*k1bar; delta2*k2bar]; %vector of steady state values for (z,e,k1,k2,I1,I2)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% No adjustment cost: Full Information LQG control
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
phi1=0;    %if both are exactly zero then there are no endogenous states (tracking problem degenerates)
phi2=0;   

%% Build LQG model parameter 
param=[alpha;theta;beta;delta1;delta2; phi1; phi2; rhoz; rhoe; tau; chi];

[Q,R,S] = QRS(param,SS,nst); %Call function to derive quadratic approximation to the objective function

%defining law of motion
A = [1 0 0 0 0;
    0 rhoz 0 0 0;
    0 0 rhoe 0 0;
    0 0 0 1-delta1 0;
    0 0 0 0 1-delta2];

B = [0 0; 0 0; 0 0; 1 0; 0 1];

% Solution under Full information
P0 = eye(nst,nst);
[P,F,err,iter] = LQG(A,B,S,Q,R,beta,P0); % Call a function to compute Riccati equation

% Remove the constant state
XX = F'*(R+beta*B'*P*B)*F;
Omega = XX(2:end,2:end);
P = P(2:end,2:end);
W = [sigmaz^2 0 0 0;0 sigmae^2 0 0;0 0 sigmak1^2 0;0 0 0 sigmak2^2];
A = A(2:end,2:end);
B = B(2:end,:);
F = F(:,2:end);  % truncate constant state

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RI solutions: No Adj
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

lambdav=[0.0001,0.0003,0.0005]; % Three information cost values
%lambdav=100*lambdav;


disp('*********************************************')
disp('RI solutions with no adjustment costs')
disp('*********************************************')

%% Case 1 (No Adj) low information cost, 

lambda= lambdav(1); %lambdav(1);

if algo == 1;
    Sigma_pr0 = A*A'+W; %Initial guess of optimal prior covariance matrix 
    Theta0 = Omega; % Initial guess of dynamic benefit matrix
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0); 
end

if algo ==2
    %cvx_solver sedumi   
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,1); %Golden rule solution with beta =1;
    Sigma_pr0 = Sigma_prGR; %
    Theta0 = ThetaGR;
    tol =1e-6;
    a = 1;
   % Modified VFI method based on a sequence of static RI problems
    [SigmaSS,Sigma_prSS,ThetaSS, Lambda] = RI_SS_SQ(A,W,Omega,lambda, beta,Sigma_pr0,Theta0,tol,a,0);    
end

if algo ==3
    %cvx_solver sedumi   
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,1); %Golden rule solution with beta =1;
    
    tol =1e-6;
    a = 1;
   % Modified VFI method using the envelope condition
    Sigma0 = SigmaGR;
    Lambda0 = LambdaGR;
    [SigmaSS,Sigma_prSS, ThetaSS, Lambda] = RI_SS_EC(A,W,Omega,lambda,beta, Sigma0, Lambda0,tol,a,0);
end

[C,V,r] = RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda); %Compute signal structure
display('Case 1: No Adj')
dimension1(1)=r;
display('Steady state signal structure for the case with no adj cost and lambda =0.0001: [C V]')
[C,V]

%impulse responses

Period = 20;
[X, u1, u_full] = RI_IRF(A,B,C,F,SigmaSS,V,W,Period);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Case 2 (No Adj) Intermediate information cost

lambda=lambdav(2);

if algo == 1
    
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0); 
end

if algo ==2
    % cvx_solver sedumi   
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,1); %Golden rule solution with beta =1;
    Sigma_pr0 = Sigma_prGR; %
    Theta0 = ThetaGR;
    tol =1e-6;
    a = 1;
   % Modified VFI method based on a sequence of static RI problems
    [SigmaSS,Sigma_prSS,ThetaSS, Lambda] = RI_SS_SQ(A,W,Omega,lambda, beta,Sigma_pr0,Theta0,tol,a,0);
end

if algo ==3
    %cvx_solver sedumi   
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,1); %Golden rule solution with beta =1;
 
    tol =1e-6;
    a = 1;
   % Modified VFI method using the envelope condition
    Sigma0 = SigmaGR;
    Lambda0 = LambdaGR;
    [SigmaSS,Sigma_prSS, ThetaSS, Lambda] = RI_SS_EC(A,W,Omega,lambda,beta, Sigma0, Lambda0,tol,a,0);
end

[C,V,r] = RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda);

display('Case 2: No Adj')
dimension1(2)=r;
display('Steady state signal structure for the case with no adj cost and lambda =0.0003: [C V]')
[C,V]

%impulse response
Period = 20;
[X, u2, u_full] = RI_IRF(A,B,C,F,SigmaSS,V,W,Period);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% high information cost, generate one dimensional signal

%% Case 3 (No Adj)

lambda=lambdav(3);
if algo ==1
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0); 
end
if algo == 2
    % cvx_solver sedumi   
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,1); %Golden rule solution with beta =1;
    Sigma_pr0 = Sigma_prGR; %
    Theta0 = ThetaGR;
    tol =1e-6;
    a = 1;
   % Modified VFI method based on a sequence of static RI problems
    [SigmaSS,Sigma_prSS,ThetaSS, Lambda] = RI_SS_SQ(A,W,Omega,lambda, beta,Sigma_pr0,Theta0,tol,a,0);
end

if algo ==3
    %cvx_solver sedumi   
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,1); %Golden rule solution with beta =1;
    
    tol =1e-6;
    a = 1;
    % Modified VFI method using the envelope condition
    Sigma0 = SigmaGR;
    Lambda0 = LambdaGR;
    [SigmaSS,Sigma_prSS, ThetaSS, Lambda] = RI_SS_EC(A,W,Omega,lambda,beta, Sigma0, Lambda0,tol,a,0);
end

[C,V,r] = RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda);

display('Case 3: No Adj')
dimension1(3)=r;
display('Steady state signal structure for the case with no adj cost and lambda =0.0005: [C V]')

[C,V]

%impulse response
Period = 20;
[X, u3, u_full] = RI_IRF(A,B,C,F,SigmaSS,V,W,Period);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Figure: Top two panels No adjustment cost
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
time=1:Period;
  
 figure(1)
    fon=7; %font size
    
    i1_imp = subplot(2,2,1);
    
    p1=plot(time,u_full(1,:,1)/(delta1*k1bar),'-.b','LineWidth',1.5);
    hold on
    p2=plot(time,u1(1,:,1)/(delta1*k1bar),':g','LineWidth',1.5);
    hold on
    p3=plot(time,u2(1,:,1)/(delta1*k1bar),'-r','LineWidth',1.5);
    hold on
    p4=plot(time,u3(1,:,1)/(delta1*k1bar),'--k','LineWidth',1.5);
    
    title('$I_{1t}$ to $\epsilon_{zt}$ (No Adj)','Fontsize',10,'interpreter','latex');
    %xlabel('Periods','interpreter','latex')
    % ylabel('(%)','Rotation',0,'interpreter','latex');
    lgd=legend([p1 p2 p3 p4],'Full Infor',strcat('\lambda =',num2str(lambdav(1))),strcat('\lambda =',num2str(lambdav(2))),strcat('\lambda =',num2str(lambdav(3))));
    lgd.FontSize=fon;
    
    xlim([0 15]);
    
    i2_imp = subplot(2,2,2);
    
    p1=plot(time,u_full(2,:,1)/(delta2*k2bar),'-.b','LineWidth',1.5);
    hold on
    p2=plot(time,u1(2,:,1)/(delta2*k2bar),':g','LineWidth',1.5);
    hold on
    p3=plot(time,u2(2,:,1)/(delta2*k2bar),'-r','LineWidth',1.5);
    hold on
    p4=plot(time,u3(2,:,1)/(delta2*k2bar),'--k','LineWidth',1.5);
    
    title('$I_{2t}$ to $\epsilon_{zt}$ (No Adj)','Fontsize',10,'interpreter','latex');
    %xlabel('Periods','interpreter','latex')
    %ylabel('(%)','Rotation',0,'interpreter','latex');
    lgd=legend([p1 p2 p3 p4],'Full Infor',strcat('\lambda =',num2str(lambdav(1))),strcat('\lambda =',num2str(lambdav(2))),strcat('\lambda =',num2str(lambdav(3))));
     lgd.FontSize=fon;   
    xlim([0 15]);

  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Ajustment costs, Full Information LQG control
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   phi1 = 0.46;   %adjustment cost case for tangible capital
   phi2 = 1.40;   %adjustment cost case for intangible capital

%% Build LQG model parameters

param=[alpha;theta;beta;delta1;delta2;phi1;phi2;rhoz;rhoe;tau;chi];

[Q,R,S] = QRS(param,SS,nst); %quadratic approximation to the objective function

%defining law of motion
A = [1 0 0 0 0;0 rhoz 0 0 0;0 0 rhoe 0 0;0 0 0 1-delta1 0;0 0 0 0 1-delta2];
B = [0 0;0 0;0 0;1 0;0 1];

% Solution under Full information
P0 = eye(nst,nst);
[P,F,err] = LQG(A,B,S,Q,R,beta,P0);


%weighting matrix OMEGA and variance-covariance matrix W 
XX = F'*(R+beta*B'*P*B)*F;
% truncate constant state
Omega = XX(2:end,2:end);
P = P(2:end,2:end);
A = A(2:end,2:end); % truncate constant state
B = B(2:end,:);  % truncate constant state
F = F(:,2:end);  % truncate constant state
W = [sigmaz^2 0 0 0;0 sigmae^2 0 0;0 0 sigmak1^2 0;0 0 0 sigmak2^2];

disp('*******************************************************')
disp('RI solutions with adjustment costs')
disp('*******************************************************')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Case 1. (Adj) Low information cost or high capacity
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
display('Case 1: Adj')
lambda=lambdav(1);

if algo == 1;
    Sigma_pr0 = W; %A*A'+W;
    Theta0 = Omega;
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0); 
    ThetaT = ThetaSS;
    Sigma0 =  W; 
    disp('Transition dynamics of information acquisition')
    T = 40; %Solve for transition dynamics
    [Sigmat, Sigma_prt, Thetat, Lambdat] = RI_TD_FOC(lambda,beta,Omega, A, W, Sigma0, ThetaT,Sigma_pr0,T);
    Sigmat1=Sigmat;
    %Dynamics of Signal Structure
    for t=1:T-1;
        [C,V,dim1(t)] = RI_SIG(Sigmat(:,:,t),Sigma_prt(:,:,t),Thetat(:,:,t),lambda);
        %[C,V,dim1(t)] = RI_SIG1(Sigmat(:,:,t),Sigma_prt(:,:,t));
        if dim1(t)==0
            disp(['t = ',num2str(t-1)])
        elseif dim1(t) ==1
            disp(['One dimension, ' ' t = ',num2str(t-1)])
            C1(:,:,t) = C;
            V1(:,:,t)=V;
        elseif dim1(t) == 2
            disp(['Two dimensions, ' ' t = ',num2str(t-1)])
            C2(:,:,t)=C;
            V2(:,:,t)=V;
        end
    end
end

if algo == 2 
    % cvx_solver sedumi   
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,1); %Golden rule solution with beta =1;
    Sigma_pr0 = Sigma_prGR; %
    Theta0 = ThetaGR;
    tol =1e-6;
    a = 1;
   % Modified VFI method based on a sequence of static RI problems
    [SigmaSS,Sigma_prSS,ThetaSS, Lambda] = RI_SS_SQ(A,W,Omega,lambda, beta,Sigma_pr0,Theta0,tol,a,0);
end

if algo ==3
    %cvx_solver sedumi   
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,1); %Golden rule solution with beta =1;
   
    tol =1e-6;
    a = 1;
    % Modified VFI method using the envelope condition
    Sigma0 = SigmaGR;
    Lambda0 = LambdaGR;
    [SigmaSS,Sigma_prSS, ThetaSS, Lambda] = RI_SS_EC(A,W,Omega,lambda,beta, Sigma0, Lambda0,tol,a,0);
end
%Compute steady-state signal structur
[C,V,r] = RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda); 

dimension2(1)=r;
display('Steady-state signal structure for the case with adj cost and lambda =0.0001: [C V]')
[C,V]

%impulse response
Period = 20;
[X, u1, u_full] = RI_IRF(A,B,C,F,SigmaSS,V,W,Period);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Case 2. (Adj) Intermediate capacity or information cost

disp('------------------------------------------------------')
display('Case 2: Adj')

lambda = lambdav(2);
if algo ==1
    Sigma_pr0 = W; %A*A'+W;
    Sigma0 = W;
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0); 
    ThetaT = ThetaSS;
    
    disp('Transition dynamics of information acquisition')
    T = 40;
    [Sigmat, Sigma_prt, Thetat, Lambdat] = RI_TD_FOC(lambda,beta,Omega, A, W, Sigma0, ThetaT,Sigma_pr0,T);
    %Dynamics of Signal Structure
    for t=1:T-1;
        [C,V,dim2(t)] = RI_SIG(Sigmat(:,:,t),Sigma_prt(:,:,t),Thetat(:,:,t),lambda);
        if dim2(t)==0
            disp(['t = ',num2str(t-1)])
        elseif dim2(t) ==1
            disp(['One dimension, ' ' t = ',num2str(t-1)])
            C1(:,:,t) = C;
            V1(:,:,t)=V;
        elseif dim2(t) == 2
            disp(['Two dimensions, ' ' t = ',num2str(t-1)])
            C2(:,:,t)=C;
            V2(:,:,t)=V;
        end
    end
end  

if algo == 2
    % cvx_solver sedumi   
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,1); %Golden rule solution with beta =1;
    Sigma_pr0 = Sigma_prGR; %
    Theta0 = ThetaGR;
    tol =1e-6;
    a = 1;
   % Modified VFI method based on a sequence of static RI problems
    [SigmaSS,Sigma_prSS,ThetaSS, Lambda] = RI_SS_SQ(A,W,Omega,lambda, beta,Sigma_pr0,Theta0,tol,a,0);
end

if algo ==3
    %cvx_solver sedumi   
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,1); %Golden rule solution with beta =1;
   
    tol =1e-6;
    a = 1;
    % Modified VFI method using the envelope condition
    Sigma0 = SigmaGR;
    Lambda0 = LambdaGR;
    [SigmaSS,Sigma_prSS, ThetaSS, Lambda] = RI_SS_EC(A,W,Omega,lambda,beta, Sigma0, Lambda0,tol,a,0);
end

[C,V,r] = RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda);

dimension2(2)=r;
display('Steady-state signal structure for the case with adj cost and lambda =0.0003: [C V]')
[C,V]

%impulse response
Period = 20;
[X, u2, u_full] = RI_IRF(A,B,C,F,SigmaSS,V,W,Period);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Case 3. (Adj) High cost or low capacity
disp('-----------------------------------------------------')
display('Case 3: Adj')
lambda = lambdav(3);

if algo == 1
    Sigma_pr0 = W; %A*A'+W;
    Sigma0 = W;
    %Steady state
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0); 
    ThetaT = ThetaSS;
    
    disp('Transition dynamics of information acquisition')
    T = 60;
    [Sigmat, Sigma_prt, Thetat, Lambdat] = RI_TD_FOC(lambda,beta,Omega, A, W, Sigma0, ThetaT,Sigma_pr0,T);
  
    %Dynamics of Signal Structure
    for t=1:T-1;
        [C,V,dim3(t)] = RI_SIG(Sigmat(:,:,t),Sigma_prt(:,:,t),Thetat(:,:,t),lambda);
        if dim3(t)==0
            disp(['t = ',num2str(t-1)])
        elseif dim3(t) ==1
            disp(['One dimension, ' ' t = ',num2str(t-1)])
            C1(:,:,t) = C;
            V1(:,:,t)=V;
        elseif dim3(t) == 2
            disp(['Two dimensions, ' ' t = ',num2str(t-1)])
            C2(:,:,t)=C;
            V2(:,:,t)=V;
        end
    end
end

if algo == 2 
    % cvx_solver sedumi   
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,1); %Golden rule solution with beta =1;
    Sigma_pr0 = Sigma_prGR; %
    Theta0 = ThetaGR;
    tol =1e-6;
    a = 1;
   % Modified VFI method based on a sequence of static RI problems
    [SigmaSS,Sigma_prSS,ThetaSS, Lambda] = RI_SS_SQ(A,W,Omega,lambda, beta,Sigma_pr0,Theta0,tol,a,0);
    
end

if algo ==3
    %cvx_solver sedumi   
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,1); %Golden rule solution with beta =1;
  
    tol =1e-6;
    a = 1;
    % Modified VFI method using the envelope condition
    Sigma0 = SigmaGR;
    Lambda0 = LambdaGR;
    [SigmaSS,Sigma_prSS, ThetaSS, Lambda] = RI_SS_EC(A,W,Omega,lambda,beta, Sigma0, Lambda0,tol,a,0);
end
        
[C,V,r] = RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda);

dimension2(3)=r;
display('Steady state signal structure for the case with adj cost and lambda =0.0005: [C V]')
[C,V]

%impulse response
Period = 20;
[X, u3, u_full] = RI_IRF(A,B,C,F,SigmaSS,V,W,Period);

disp('Steady state signal dimension: No adj case')
dimension1
disp('Steady state signal dimension: Adj case')
dimension2

toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Figure: Bottom two panels with adjustment costs
    
   figure(1) 
    i1_imp = subplot(2,2,3);
    
    p1=plot(time,u_full(1,:,1)/(delta1*k1bar),'-.b','LineWidth',1.5);
    hold on
    p2=plot(time,u1(1,:,1)/(delta1*k1bar),':g','LineWidth',1.5);
    hold on
    p3=plot(time,u2(1,:,1)/(delta1*k1bar),'-r','LineWidth',1.5);
    hold on
    p4=plot(time,u3(1,:,1)/(delta1*k1bar),'--k','LineWidth',1.5);
    
    title('$I_{1t}$ to $\epsilon_{zt}$ (Adj)','Fontsize',10,'interpreter','latex');
    xlabel('time','interpreter','latex')
    % ylabel('(%)','Rotation',0,'interpreter','latex');
    lgd=legend([p1 p2 p3 p4],'Full Infor',strcat('\lambda =',num2str(lambdav(1))),strcat('\lambda =',num2str(lambdav(2))),strcat('\lambda =',num2str(lambdav(3))));
    lgd.FontSize=fon;
    
    xlim([0 15]);
    
    i2_imp = subplot(2,2,4);
    
    p1=plot(time,u_full(2,:,1)/(delta2*k2bar),'-.b','LineWidth',1.5);
    hold on
    p2=plot(time,u1(2,:,1)/(delta2*k2bar),':g','LineWidth',1.5);
    hold on
    p3=plot(time,u2(2,:,1)/(delta2*k2bar),'-r','LineWidth',1.5);
    hold on
    p4=plot(time,u3(2,:,1)/(delta2*k2bar),'--k','LineWidth',1.5);
    
   
    %xlabel('Periods','interpreter','latex')
    % ylabel('(%)','Rotation',0,'interpreter','latex');
    lgd=legend([p1 p2 p3 p4],'Full Infor',strcat('\lambda =',num2str(lambdav(1))),strcat('\lambda =',num2str(lambdav(2))),strcat('\lambda =',num2str(lambdav(3))));
    lgd.FontSize=fon;
   
    
    title('$I_{2t}$ to $\epsilon_{zt}$ (Adj)','Fontsize',10,'interpreter','latex');
    xlabel('time','interpreter','latex','Fontsize',10)
  %  ylabel('(%)','Rotation',0,'interpreter','latex');
    lgd=legend([p1 p2 p3 p4],'Full Infor',strcat('\lambda =',num2str(lambdav(1))),strcat('\lambda =',num2str(lambdav(2))),strcat('\lambda =',num2str(lambdav(3))));
     lgd.FontSize=fon;
     
    xlim([0 15]);


print -depsc Fig7New.eps

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions called in the main code
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Q, R, S] = QRS(param,SS,nst)
%Compute quadratic approximation to the objective function
 % SS is the vector of steady state values for the states (excluding constant state 1)  and controls
 % SS=[ 0.0; 0.0; k1bar; k2bar; delta1*k1bar; delta2*k2bar];
 % nst is the number of state variables including the constant state 1
 
   [f,Df,D2f] = num_derivative(SS,param);
   
    M11 = f -Df'*SS + 0.5*SS'*D2f*SS;
    M12 = 0.5*(Df' - SS'*D2f);
    M22 = 0.5*D2f;
    M = [M11 M12; M12' M22];
    
    %In MWY (2020) notation
    Q = -M(1:nst,1:nst); 
    S = -M(1:nst,nst+1:end);
    R = -M(nst+1:end, nst+1:end);
end

function [nf, nDf nD2f] = num_derivative(x,p)
% x is the input steady-state value
% p is the vector of parameters

 alpha=p(1); theta=p(2); beta=p(3); deltat=p(4); deltai= p(5); phi1= p(6); 
 phi2=p(7); rhoz=p(8); rhoe=p(9); tau=p(10); chi = p(11);
 
%Need Matlab symbolic toolbox
syms z e kt ki It Ii 

 f = exp(z+e)*kt^alpha*ki^theta-tau*(exp(z+e)*kt^alpha*ki^theta-chi*Ii)-Ii-It-...
        phi1/2*(It/kt-deltat)^2*kt-phi2/2*(Ii/ki-deltai)^2*ki;%dividend function
 Df = gradient(f,[z, e, kt, ki, It,Ii]); %gradient
 D2f = hessian(f,[z, e, kt, ki, It, Ii]); %Hessian
    z = x(1); e = x(2); kt = x(3); ki=x(4); It = x(5); Ii = x(6);
    nf = double(subs(f)); %eval(f);
    nDf = double(subs(Df)); %eval(Df);
    nD2f = double(subs(D2f)); %eval(D2f);
end