%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This code solves for the steady state information structure for the pricing example in
%Sims's hanndbook chapter using (i) the modified value function iteration
%using envelope condition, (ii) the modified VFI method based on a sequence
%of static RI problems, (iii) Afrouzi and Yang (2019) method, (iv) MWY
%method based on FOCs, and (v) golden rule solution with beta =1
%
% We illustrate that AY Matlab code contains a bug that may generate a wrong solution. Their
% code is sensistive to the initial condition and parameter values. 
%
% You need to install the software CVX that can be freely 
% downloaded from the website: http://cvxr.com/cvx/
%
%  @Written by Jianjun Miao Email: miaoj@bu.edu
% Department of Economics, Boston University
%
%% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all
close all


%% Generate parameter data


rho_1 = 0.95; %more persistent state
rho_2 = 0.40;
beta = 0.9;  %Discount factor

Q =[sqrt(1-rho_1^2) 0; 0 sqrt(1-rho_2^2)]; %The Q matrix in Afrouzi and Yang (2019)
%Q =[sqrt(1-rho_1^2) 0; 0 sqrt(0.86)];
%Q = [sqrt(0.0975) 0; 0 sqrt(0.84)];
W = Q*Q'; %State noise variance matrix

A = [rho_1,0; %State transition matrix
    0,rho_2];

G = [1,1];

Omega = G'*G; %Benefit matrix of information
% Information cost parameter. My lambda corresponds to 2*lambda in Sims
% handbook example and 2*omega in Afrouzi and Yang (2020).
% When lambda >=4, AY method typically converges to wrong solutions for any
% value of beta.
lambda = 4; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cvx_solver mosek %sedumi

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Afrouzi and Yang (2019) method converges to wrong solution when lambda >= 4. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
H = G';
Omega_init = Omega; %AY's default initial guess
Sigma_init = A*A'+W; %AY's default initial guess. If setting to W, the code generates the right solution.
[Sigma_prAY,SigmaAY,Lambda,Omega0,Y,Sigma_z,K] = Solve_RI_Dynamics(lambda/2,beta,A,Q,H,Omega_init,Sigma_init);
tAY = toc;


disp(['AY solution time (seconds): ' num2str(tAY)])
SigmaAY


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Our method based on FOC
%
    tic
    tol = 1e-10;
    Sigma_pr0 = A*A'+W;
    Theta0 = Omega;
    [SigmaFOC, Sigma_prFOC, ThetaFOC, LambdaFOC] = RI_SS_FOC(lambda,beta,A,W, Omega, Theta0, Sigma_pr0,tol);    
    tFOC = toc;

disp(['Our solution based on FOC time (seconds): ' num2str(tFOC)])
SigmaFOC
CSC = trace(LambdaFOC*(Sigma_prFOC-SigmaFOC));
disp(['Check complementary slackness condition:  ' num2str(CSC)]);
FOC = lambda/2*inv(SigmaFOC) - ThetaFOC - LambdaFOC;
disp(['Check FOC:  ' num2str(norm(FOC))])

    disp('-------------------------------------------')
    disp('Compute signal structure using two algorithms')
[C,V,r] = RI_SIG(SigmaFOC,Sigma_prFOC,ThetaFOC,lambda);
CN = C/C(2);
VN = V/C(2)^2;
disp('Algorithm 1. Our FOC optimal signal structure [C V]')
[CN VN]

disp('Algorithm 2. Our FOC optimal signal structure: [C, V]')
[C1,V1,r1] = RI_SIG1(SigmaFOC,Sigma_prFOC); %
[C1(1)/C1(2),1, V1/C1(2)^2]


%
%%Golden rule solution when beta =1
%
tic
[SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0,lambda,1);
tGR = toc;
disp(['Golden Rule solution time (seconds): ' num2str(tGR)])
SigmaGR

%
%%Our preferred VFI method based on a sequence of static RI problems
%
tstart = tic;
Theta0 = Omega;
Sigma_pr0 = A*A'+W; %Sigma_prGR; %
a =1; %Updating weight on the new iteration
tol=1e-6; %Error tolerance
[SigmaSQ,Sigma_prSQ,ThetaSQ, LambdaSQ] = RI_SS_SQ(A,W,Omega,lambda, beta,Sigma_pr0,Theta0,tol,a);
tendSQ = toc(tstart); 
disp(['Our MVFI based on sequence of static RI problems time (seconds): ' num2str(tendSQ)])
 SigmaSQ
%
%% Modified VFI method using envelope condition
%
Sigma0 = SigmaGR; %Using the golden rule solution as the initial guess
Lambda0=LambdaGR; 
tic
[SigmaEC,Sigma_prEC, ThetaEC, LambdaEC] = RI_SS_EC(A,W,Omega,lambda,beta, Sigma0, Lambda0);
tEC=toc;
disp(['Our MVFI method using envelope condition time (seconds): ' num2str(tEC)])
SigmaEC


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



