%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program solves for the golden rule information structure (steady state for beta = 1)
% for tracking problem with per-by-period capacity constraints 
% based on Sims Handbook Example. See Appendix B of Miao, Wu, and Young (2020)
% You need to install the software CVX that can be freely 
% downloaded from the website http://cvxr.com/cvx/.
% 
% Call a Matlab function: RI_SIG1.m
%
% @Written by Jianjun Miao Email: miaoj@bu.edu
% Department of Economics, Boston University
%
%% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



clear all
close all

tic
%generate data

rho_1 = 0.95; %more persistent state
rho_2 = 0.4;
beta = 0.9; %Solution is independent of beta

W=[0.0975 0;
    0 0.84];

A=[rho_1,0;
    0,rho_2];

a=[1,1]';
Omega=a*a';
kappa= 0.2;  %Information capacity parameter

n = size(A,1);

cvx_solver mosek
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Golden rule 
%Create and solve problem

cvx_begin sdp quiet 
   variable Sigma1(n,n) semidefinite  
   minimize( trace(Omega*Sigma1)) 
 subject to
        A*Sigma1*A'+W >= Sigma1;
        L(Sigma1,A,W) <= 2*kappa;
cvx_end

disp('Golden rule posterior covariance matrix')
SigmaGR = Sigma1
Sigma_prGR = A*SigmaGR*A'+W;
disp('Check no-forgetting constraint')
B = Sigma_prGR-SigmaGR;
eig(B) %Check if no-forgetting constraint binds

%disp('Signal-to-Noise Ratio')
SNR = inv(SigmaGR)-inv(Sigma_prGR); 

[C,V]= RI_SIG1(SigmaGR,Sigma_prGR); %Signal dimension = rank(SNR)
CN=C/C(1);
VN = V/C(1)^2;

disp('Golden rule signal structure [C,V]')
[CN, VN]  %check CN = a'??


toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function used in the capacity constraint
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function cvx_optval = L(X,A,W) 
%Assume W is invertible
n = size(A,1);
cvx_begin sdp quiet
   variable PI(n,n) semidefinite 
   minimize( log_det(W)-log_det(PI) )
   subject to
        [X-PI, X*A';
            A*X, A*X*A'+W]>=0;
cvx_end
end


