%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program solves the steady state information structure under RI with
% discounted information costs for the GE pricing example in Section 5.1 of Miao, Wu, and Young (2020).
% The model is adapted from Mackowiak and Wiederholt (2009) without strategic complementarity
% 
% You need to call matlab functions RI_SS_FOC.m, RI_SIG.m, armafit.m, Varma2ma1.m
%
%% @ Written by Jianjun Miao. Email: miaoj@bu.edu
% Department of Economics, Boston University
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic

global lambda rho_1 rho_2 sigma_1 sigma_2 alpha_2 alpha_3 beta
 
 %Assign parameter values

%rho_1 = 0.95;% 0.95;%0.9; % persistence of idiosyncratic shock
%rho_2 = 0.95; %0.95;%0.8; % persistence of aggregate demand shock
%sigma_1 = 1; % S.D. of idiosyncratic innovation
%sigma_2 = 0.1; % S.D. of aggregate innovation
%alpha_2 = 1; % No strategic complementarity
%alpha_3 = 1;
%lambda = .2; %0.1;  %0.8;%0.001; information cost
%beta = 0.95;

W=[sigma_1^2 0; %Shock variance
    0 sigma_2^2];

A=[rho_1,0;
    0,rho_2];

G=[alpha_3,1];
Omega=G'*G;

%Steady State Solution 
%Create and solve problem

 Tol_temp = 1e-6; %Adjustable error tolernance 
 a_temp = 0.2; %Adjustable updating weight
    
    Sigma_pr0 = A*A'+W; %Initial guess for prior covariance matrix at the first iteration
    Theta0 = Omega;  %Initial guess for dynamic benefit matrix at the first iteration
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0,Tol_temp, a_temp); 


%Signal structure
[C,V,r]=RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda); %Signal dimension = rank(SNR)
K = Sigma_prSS*C'/(C*Sigma_prSS*C'+V); %Kalman gain
KC = K*C;


z=-0.99:0.01:0.99;
N=length(z);
M=[0,1]';
Psi = zeros(N,1);

%Aggregate price process p_t=\Psi(L)\epsilon_{a,t}
for i=1:N;
X=inv(eye(2)-(eye(2)-K*C)*A*z(i));
Psi(i)=G*X*KC*inv(eye(2)-A*z(i))*M;
fz(1,1,i)=Psi(i);
end;


k=10; %Maximum ARMA(k,k) order
tol1 = 1e-7;
[av,bv] = armafit(z,fz,k,k,tol1); %fit an ARMA(p,q) function to data
p=length(av);
q=length(bv);
a=zeros(1,p);
b=zeros(1,q);
for i=1:p;
a(i)=av(1,1,i);
end
for i=1:q;
b(i)=bv(1,1,i);
end

disp(['AR coefficients a: ' num2str(a)])
disp(['MA coefficients b: ' num2str(b)])
disp(['AR order p: ' num2str(p) ';  MA order q: ' num2str(q-1)])
disp('Signal structure: [C V]')
[C V]
ar0 = LagOp([1 -a]);
ma0 = LagOp(b);

%%% Compute the individual price representation
pz = zeros(1,2,N);  %% consistent with RI04 notation, eta_{it} dimension is augmented (B multiplies with true structural innovations)
pi = zeros(1,1,N);

for i = 1:N
    X=pinv(eye(2)-(eye(2)-K*C)*A*z(i));
    pz(:,:,i) = G*X*KC*pinv(eye(2)-A*z(i));
    pi(:,:,i) = pz(1,1,i); %% first elements in eta_{it} is idiosyncratic shock, only the first one
end

[ai,bi] = armafit(z,pi,10,10,tol1); %% unrestricted ARMA expansion
maexpan_pi = Varma2ma1(ai,bi,200);
maexpan_pi = maexpan_pi(1,1:100);


%% Plot impluse response functions

%figure
%y1=armairf(ar0,ma0,'NumObs',20);
%t=0:19;
%plot(t,rho_2.^t,'--',t,y1)
%lgd=legend('Full Infor','\lambda = 0.002','Location','northeast');
%lgd.FontSize=7;
%xlabel('Periods')


toc
