%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code compute the GE pricing model in Section 5.1 of Miao, Wu, and Young (2020).
% The model is adapted from Mackowiak and Wiederholt (2009) with discounted information costs
% Solve for the steady state information structure without the signal independence assumption
% You need to install the code armafit.m
% You need to install the software CVX for algorithms 1 and 3 that can be freely 
% downloaded from the website:
% http://cvxr.com/cvx/
%
% @ Written by Jianjun Miao, Boston University, Email: miaoj@bu.edu 
%           and Jieran Wu, Zhejiang University, Email: jw5ya@zju.edu.cn
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% close all
% clear all
global lambda rho_1 rho_2 sigma_1 sigma_2 alpha_2 alpha_3 beta
tic
 
%*******************************************************************************************
%******************************  PRELIMINARY CALIBRATION        ******************************************
%*******************************************************************************************

%Predetermined Structural Parameter using MW(2009)

 %rho_1 = 0.95; %0.4; % persistence of idiosyncratic shock
 %rho_2 = 0.95; % persistence of aggregate demand shock
 %sigma_1 = 1.0; % Scaled S.D. of idiosyncratic innovation
 %sigma_2 = 0.1; % Scaled S.D. of aggregate innovation 
 %alpha_2 = 0.15; %0.15 %profit-maximizing price reponse to aggregate shock
 %alpha_3 = 1; % profit-maximizing price reponse to idiosyncratic shock
 %lambda = 0.2; %0.1;  %0.8;%0.001; information cost
 %beta = 0.95; 
 
algorithm = 2; % If algorithm = 1 (stable), use the FOC based method and the golden-rule solution as the initial guess. 
               % If algorithm = 2 (fasest), use the FOC based method and the solution from the previous iteration as the guess for the current iteration.
               % If algorithm = 3 (fastest), use AY (2020) toolbox to get the same results.                                
               % If algorithm = 4 (slowest), use the MVFI method based on envelope condition and use the golden-rule solution as the starting guess 
                                  %in the first iteration and then use th previous iteration as the guess for the current iteration. 
                                  %Set epsilon1=10^-3 to get convergence.


epsilon1=10^(-4); %Tolerance level for convergence of pricing function. Set as 10^(-3) for algorithm 4
w = 0.6; %0.4 %Updating weight for the ARMA coefficients for the equilibrium price 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Step 0. Initialize k and fix a grid 
k =2; %maximum AR order
z=-0.99:0.01:0.99; %Grid points
N=length(z);

while k<10;  %Outer loop
    
%Step 1. Initialize polynomial coefficients a, b
p=k; %Endogenous AR order 
q=k; %Endogenous MA order is q-1 
a=0.01*ones(1,p); %AR coefficients
b=0.01*ones(1,q); %MA coefficients

%Initialize state space representation

A=zeros(p+2,p+2);
G=zeros(1,p+2);
W=zeros(p+2,p+2);
W(1,1)=sigma_1^2;
W(2,2)=sigma_2^2;
W(2,3)=sigma_2^2;
W(3,2)=sigma_2^2;
W(3,3)=sigma_2^2;
 
iter=0;
Diff=1;
while Diff>epsilon1 & iter< 300;  %Inner loop

    %Step 2. Compute \Psi(z_i)

    for i=1:N;
    denom=1;
    numer=0;
        for j=1:p;
        denom=denom-a(j)*z(i)^j;
        end
        for m=1:q;
        numer=numer+b(m)*z(i)^(m-1);
        end
    Psi(i)=numer/denom;  %Rational function
    end;

    % Step 3. Construct the state space representation
    A=zeros(p+2,p+2); %AR order p is endogenous
    G=zeros(1,p+2);
    W=zeros(p+2,p+2); 
    W(1,1)=sigma_1^2;
    W(2,2)=sigma_2^2;
    W(2,3)=sigma_2^2;
    W(3,2)=sigma_2^2;
    W(3,3)=sigma_2^2;
    A(1,1)=rho_1;
    A(2,2)=rho_2;
    A(3,3:end)= a;
    for i=1:p-1;
    A(3+i,2+i)=1;
    end;
    
    G=[alpha_3,alpha_2,(1-alpha_2)*b]; %When alpha_2 = 1 the dimesion of matrix G may cause error. Instead set alpha_2=0.999.

if rank(A) < p+2 
    error('A is not invertible');
    return
end

% Compute Steady State RI solution
Omega = G'*G; 

if algorithm == 1; % Use the golden rule solution as the initial guess for the steady state  
    disp(['Algrithm ' num2str(algorithm) ': FOC based, GR rule as initial guess']);
   %Use default SDPT3 solver, which is slower, but more reliable and more stable
   cvx_solver sedumi %Mosek is the fastest, but does not converge when alpha2=1
   [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,beta);
    
    Sigma_pr0 = Sigma_prGR; %Initial guess for prior
    Theta0 = ThetaGR; % Initial guess of dynamic benefit matrix
    Tol_temp = 1e-7; %Set error tolernance 
    a_temp = 0.3; %Set updating weight
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0,Tol_temp, a_temp); 
    
end

if algorithm == 2; %Use solution in the previous iteration as initial guess for the current iteration
    disp(['Algrithm ' num2str(algorithm) ': FOC based, previous solution as initial guess']);
    Tol_temp = 1e-7; %6; %Adjustable error tolernance 
    a_temp = 0.2; %Adjustable updating weight
    if iter == 0
    Sigma_pr0 = A*A'+W; %Initial guess for prior covariance matrix at the first iteration
    Theta0 = Omega;  %Initial guess for dynamic benefit matrix at the first iteration
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0,Tol_temp, a_temp); 
    
    else
    Sigma_pr0 = Sigma_prSS; %Use the solution in the previous iteration as the new guess for the current iteration
    Theta0 = ThetaSS;
    [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0,Tol_temp, a_temp); 
    end

end


if algorithm == 3; %Use AY method and use the solution in the previous iteration as initial guess for the current iteration
     disp(['Algrithm ' num2str(algorithm) ': AY method']);
    omega = lambda/2; %AY's information cost parameter
    [u,d,v] = svd(W);
    Q = u*sqrt(d);
    H = G';
    if iter == 0
    Sigma_init = A*A'+W; %Initial guess for prior covariance matrix at the first iteration
    Omega_init = Omega;  %Initial guess for dynamic benefit matrix at the first iteration
    
    [Sigma_prSS,SigmaSS,Lambda,ThetaSS,Y,Sigma_z,K] = ...
                    Solve_RI_Dynamics(omega,beta,A,Q,H,Omega_init,Sigma_init); 
    else
    Sigma_init = Sigma_prSS; %Use the solution in the previous iteration as the new guess for the current iteration
    Omega_init = ThetaSS;
    
    [Sigma_prSS,SigmaSS,Lambda,ThetaSS,Y,Sigma_z,K] = ...
                    Solve_RI_Dynamics(omega,beta,A,Q,H,Omega_init,Sigma_init);
    end

end


if algorithm == 4; %Use MVI method using envelope condition
     disp(['Algrithm ' num2str(algorithm) ': MVI method using envelope condition method']);
    cvx_solver mosek %sedumi %Mosek is the fastest
    Tol_temp = 1e-5; %Adjustable error tolernance 
    a_temp = 0.5; %Adjustable updating weight
    
    if iter == 0
        [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0, lambda,beta);
        Sigma0  = SigmaGR;   %Initial guess for prior covariance matrix at the first iteration
        Lambda0 = LambdaGR;  %Initial guess for dynamic benefit matrix at the first iteration
        [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_EC(A,W,Omega,lambda,beta, Sigma0,Lambda0,Tol_temp, a_temp,0);
        
    else
        Sigma0 = SigmaSS; %Use the solution in the previous iteration as the new guess for the current iteration
        Lambda0 = LambdaSS;      
        [SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_EC(A,W,Omega,lambda,beta, Sigma0, Lambda0,Tol_temp, a_temp,0); 
    end
    
end

KC = eye(size(SigmaSS,1))-SigmaSS/(Sigma_prSS);

%Step 4. Compute the updated \Psi function values

M = zeros(p+2,1);
M(2) = 1; M(3) = 1;
for i=1:N;
X = inv(eye(p+2)-(eye(p+2)-KC)*A*z(i));  
Psi_new(i) = G*X*KC/(eye(p+2)-A*z(i))*M;
fz(1,1,i) = Psi_new(i);
end;

%Step 5. Compute the updated polynomial coefficients or ARMA order.
tol1 = 1e-7;
[av,bv] = armafit(z,fz,k,k-1,tol1); %fit an ARMA(k,k-1) function
p1 = length(av); % endogenous AR order p1<=k
q1 = length(bv); % endogenous MA order
   if q1>p1; % MA order must be lower than AR order
        q1=p1; 
   end

a_new = zeros(1,p); %p=k
b_new = zeros(1,p);

for i=1:p1;
a_new(i) = av(1,1,i);
end
for i=1:q1;
b_new(i) = bv(1,1,i);
end

%Step 6. Compute inner loop stopping criterion

iter=iter+1;
Diff=norm(Psi_new-Psi)/norm(Psi); % Change of functional values
Text = ['Inner loop: ' num2str(iter) ', Function Error: ' num2str(Diff)];
            disp(Text); 
           
%Update ARMA coefficients
a = w*a_new+(1-w)*a; 
b = w*b_new+(1-w)*b;

end

%Final step, compute the norm difference between the MA expansions of the ARMA process obtain 
%in the previous step and the unrestricted ARMA(p,q) process

    maexpan = Varma2ma1(av,bv,200); % MA(200) representation of the ARMA price process
    maexpan = maexpan(1,1:100); %MA coefficients up to 100 order (IRF to aggregate shock)
    tol=10^(-7);    
    [au,bu] = armafit(z,fz,10,10,tol); %% ARMA function fits the values of the converged price function 
                                       % without the upper bound on the AR order.
    maexpan_new = Varma2ma1(au,bu,200);
    maexpan_new = maexpan_new(1,1:100);
    
    Diff = norm(maexpan-maexpan_new)/norm(maexpan);
   
    disp('===================================')
    disp(['(Outer loop) maximum ARMA order: ' num2str(k) ', IRF Error: ' num2str(Diff)]) 
    
    if iter<200 & Diff < epsilon1;
        disp('Equilibrium solution converges')
       
        break;
    end
disp('Increase k')
k = k+1;

 end

disp(['AR coefficients a: ' num2str(a)])
disp(['MA coefficients b: ' num2str(b)])
disp(['AR order p: ' num2str(p) ';  MA order q: ' num2str(q-1)])

[C,V,r]=RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda); %Compute signal structure 
disp('Steady state signal structure: [C, V] ')
[C,V]     

% Plot IRF of the aggregate price level 
%to a unit size innovation aggregate shock

ar0 = LagOp([1 -a]);
ma0 = LagOp(b);

%%% Compute MA  representation of the individual price
Pz = zeros(1,p+2,N); 
Pi = zeros(1,1,N);

for i = 1:N
    X = inv(eye(p+2)-(eye(p+2)-KC)*A*z(i));
    Pz(:,:,i) = G*X*KC/(eye(p+2)-A*z(i));
    Pi(:,:,i) = Pz(1,1,i); %% first elements in eta_{it} is idiosyncratic shock, only the first one
end

[ai,bi] = armafit(z,Pi,10,10,tol1); %% unrestricted ARMA expansion
maexpan_pi = Varma2ma1(ai,bi,200); %MA expansion (IRF to idiosyncratic shock)

toc

