%% This Matlab code computes the golden rule information structure for the multivariate RI problem in the LQG framework
%% The program works under the assumption that A*A'+W is invertible
% You need to install the code armafit.m
% You need to install the software CVX for algorithm 1 that can be freely 
% downloaded from the website:
% http://cvxr.com/cvx/
%% Input:
% A:      State transition matrix
% W:      State noise covariance matrix
% Omega:  One-period benefit of information
% P:      Matrix in the value function for the LQG control problem
%          For tracking problem set P=0;
% lambda: Information cost parameter
% beta:   Discount factor (for tracking problem set beta =1)

%% Ouput:
% Sigma1:   Golden rule posterior covariance matrix
% Sigma_pr: Golden rule prior covariance matrix
% Lambda:   Lagrange multiplier for the no-forgetting constraint
% Theta:    Dynamic benefit matrix of information
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by Jianjun Miao, Department of Economics, Boston University
% Emai: miaoj@bu.edu
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 function [Sigma1,Sigma_pr, Lambda, Theta] = RI_GR(A,W,Omega,P, lambda,beta)
 
    if min(eig(A*A'+W))<1e-8
    disp('Invertibility Condition is violated: Stop')
    return
    end

    n = size(A,1); %State dimension

    if min(eig(W)) > 1e-8 %If W is invertible
    disp('W is invertible: Golden rule')
    cvx_begin sdp quiet 
    variable Pi(n,n) semidefinite %Auxilary variable
    variable Sigma1(n,n) semidefinite %Posterior covariance matrix 
    dual variable Lambda   % Lagrange multiplier 
    minimize( (1-beta)*trace(A'*P*A*Sigma1) + trace(Omega*Sigma1)+...
      +0.5*lambda*(log(det(W))-log_det(Pi))) 
    subject to
      Lambda: A*Sigma1*A'+W>=Sigma1; % Lagrange multiplier for the no-forgetting constraint
        [Sigma1-Pi, Sigma1*A';
            A*Sigma1, A*Sigma1*A'+W]>=0;
   %     log_det(Sigma1)>=-0.01; %Add additional entropy constraint as an example
    cvx_end

    elseif min(abs(eig(A)))>1e-8 %If A is invertible, but W is not
    disp('A is invertible, but W is not: Golden rule')
      [U,D,V]= svd(W); 
      M = U*sqrt(D)*U'; %decompose W=M*M'
    cvx_begin sdp quiet
    variable Pi(n,n) semidefinite
    variable Sigma1(n,n) semidefinite
    dual variable Lambda
    minimize((1-beta)*trace(A'*P*A*Sigma1)+trace(Sigma1*Omega) -0.5*lambda*log_det(Pi)) %We removed the constant of log|A|
    subject to
     Lambda: A*Sigma1*A'+W>=Sigma1;       
        [eye(n)-Pi, M';
          M, A*Sigma1*A'+W]>=0;
    cvx_end
    
    else
          disp('Both A and W are singular, but A*A^p+W is invertible')
          disp('Use CCP algorithm: Golden rule') 
       
        err = 1; % initial iteration error
        tol = 1e-6; % convergence criteria
        iter = 0; % iteration count
        n=size(A,1);
        Sigmak = eye(n);

        while err > tol & iter < 1000
   
        cvx_begin sdp quiet
        variable Sigma1(n,n) semidefinite 
        dual variable Lambda
        minimize((1-beta)*trace(A'*P*A*Sigma1) + trace(Omega*Sigma1)-0.5*lambda*log_det(Sigma1)...
            +0.5*lambda*(log_det(A*Sigmak*A'+W)+trace( A'/(A*Sigmak*A'+W)*A*(Sigma1-Sigmak)))) 
        subject to
          Lambda: A*Sigma1*A'+W>=Sigma1;
        cvx_end
 
        err = norm(Sigmak-Sigma1,'fro')/norm(Sigmak,'fro');
        
            Sigmak = Sigma1;
       
          iter = iter + 1;               
          Text = ['GR CCP Iteration: ' num2str(iter) ', Error: ' num2str(err)];
          %  disp(Text);     
        end
        %Text = ['GR CCP Iteration: ' num2str(iter) ', Error: ' num2str(err)];
         disp(Text); 
    end
    
    Sigma_pr = A*Sigma1*A'+W; % Prior covariance matrix
    Theta = Omega +0.5*lambda*A'/Sigma_pr*A - A'*Lambda*A; 
end

