%% This code generates IRF to a one-standard-deviation shock given the state space representation and RI solution for the optimal signal structure 
%% in the dynamic, multivariate rational inattention (LQG) framework.
%% The state transition allows for endogenous controls, IRF computed based on Kalman filter and augmented state space
%% Controlled states: x_t = A*x_{t-1} + B*u_{t-1} + L*e_t, where e_t is a Gaussian white noise with ne dimensional identity covariance matrix
%% Signal: s_t = C*x_t + v_t
%% Control: u_t = -F*\hat{x_t}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Input:
%
% A 	     : State transition matrix of dimension n by n
% B          : Feedback matrix of control to state, dimension nxk. If pure exogenous states, specify B = zeros(n,k), k is the dimension of endogenous variable that user want to generates IRF on
% C          : Coefficient matrix of signals in terms of states, dimension rxn
% F          : Optimal control's decision rule matrix, dimension k x n, certainty equivalence holds. If exogenous states, F is speficied by users as decision rule of any endogenous variable that they want to generate IRF
% Sigma      : Optimal posterior covariance matrix obtained from RI problem,dimension nxn
% V          : Covariance matrix of endogenous signal noise, dimension rxr
% L          : Exogenous n by ne matrix coefficients of orthogonal shocks 
% Period     : Number of Periods for IRF
%
%% Output:
%
% X         : Response matrix array for augmented state vector to n structural shock and r noise shock; X_t =[x_t; x^_t]; dimension 2n x Period x (n+r) 
% u         : Response matrix array for control variable or endogenous variable u_t to n structural shock and r noise shock;  dimension k x Period x (n+r)
% u_full    : Full information response matrix array for control variable or endogenous variable u_t to n structural shock; dimension k x Period x n
%
% Written by Jianjun Miao, Boston University, email: miaoj@bu.edu
%            and Jieran Wu, Zhejiang University, email: jw5ya@zju.edu.cn
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [X, u, u_full] = RI_IRF2(A,B,C,F,Sigma,V,L,Period)

n = size(A,1); %State dimension
k = size(B,2); %Control dimension
r = size(C,1); %Signal dimension
ne = size(L,2); %Number of independent structural innovations
W = L*L';  %State noise covariance matrix

% Kalman gain matrix and auxilary matrices
K = (A*Sigma*A'+W)*C'*pinv(C*(A*Sigma*A'+W)*C'+V);
KC = K*C;
G = [eye(n), zeros(n,n); -KC, eye(n)]; % By construction, G is invertible

if B ==zeros(n,k) %Exogenous states for pure tracking problem
    Ab = [A, zeros(n,n); zeros(n,n), (eye(n)-KC)*A];
else %Control problem
    Ab = [A, -B*F; zeros(n,n), (eye(n)-KC)*(A-B*F)]; 
end

N = [L,zeros(n,r);zeros(n,ne),K]; % 2n by ne+r matrix

% Construct augmented state space representation

Aa = inv(G)*Ab;
Va = inv(G)*N; 


% Compute impulse response 

X = zeros(2*n,Period,(ne+r)); % First n dimension in X are the response of true state

X_tfull = zeros(n,Period,(ne+r)); % True state dynamics under full information, in pure tracking problem equivalent to first part of X

u = zeros(k,Period,(ne+r));

u_full = zeros(k,Period,(ne+r)); % control or endogenous decision under full information

I = eye(ne+r); % possible innovation matrix

for i = 1:ne+r     
    
    X(:,1,i) = Va*I(:,i); % One-standard-deviation innovation shock 
    u(:,1,i) = -F*X(n+1:end,1,i);  % initial response of optimal control is function of estimated states
    
    for j = 2:Period
        
        X(:,j,i)= Aa*X(:,j-1,i);
        u(:,j,i) = -F*X(n+1:end,j,i);
         
    end

end

I1 = eye(ne); % possible innovation matrix under full information

for i = 1:ne
    X_tfull(:,1,i) = L*I1(:,i);
    u_full(:,1,i) = -F*X_tfull(:,1,i);
    
    for j = 2:Period
        X_tfull(:,j,i) = (A-B*F)* X_tfull(:,j-1,i);  % Include B = 0, pure tracking case
        u_full(:,j,i) = -F*X_tfull(:,j,i);
    end
end

end