
function [C,V,r] = RI_SIG(Sigma,Sigma_pr,Theta,lambda)
% 
% This function computes the signal s = C*x + v to generate the
% posterior covariance matrix from a prior covariance matrix for the state x.
% The algorithm uses a result in MWY (2020).
%
% Input:
%    Sigma:    Posterior covariance matrix of state
%    Sigma_pr: Prior covariance matrix of state
%    Theta:    Dynamic marginal benefit matrix of information
%    lambda:   Information cost parameter
% Output:
%    C:  Signal weight matrix   
%    V:  Noise covariance matrix
%    r:  Signal dimension
%
% @ Written by Jianjun Miao, Boston University, Email: miaoj@bu.edu 
%          
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if min(eig(Sigma_pr))<1e-10
    disp('Sigular prior (smallest eigenvalue less than 1e-10): Stop')
    return
end

n = size(Sigma,1);
sqrtSigma = sqrtm(Sigma_pr);

[u,D,v] = svd(sqrtSigma*Theta*sqrtSigma);

[u,S,v] = svd(inv(sqrtSigma)*u*max(0,2/lambda*D-eye(n))*u'*inv(sqrtSigma));

% Because taking inverse can cause nontrivial numerical errors, one has to
% be cautious to use the commented approach below:
%SNR = pinv(Sigma) - pinv(Sigma_pr); %Signal to noise ratio 
%[u,S,v] = svd(SNR);

% Optimal Signal Dimension
r = 0; 

for i = 1:n
    if D(i,i) > lambda/2    
        r = r+1;
    end
end

if r == 0
    disp('No signal is acquired')
    C = 0; V = inf; r=0;
    return
end

V = inv(S(1:r,1:r)); 

Delta = [eye(r,r), zeros(r,n-r)];

C = Delta*u';
    
end

