
function [C,V,r] = RI_SIG1(Sigma,Sigma_pr)
% 
% This function computes the signal s = C*x + v to generate the
% posterior covariance matrix from a prior covariance matrix for the state x
% Using this algorithm when the prior and posterior covariance matrices are not ill-conditioned so that the inverse can be accurately computed.
%
% Input:
%    Sigma:    Posterior covariance matrix of state
%    Sigma_pr: Prior covariance matrix of state
%   
% Output:
%    C:  Signal weight matrix   
%    V:  Noise covariance matrix
%    r:  Signal dimension
%
% @ Written by Jianjun Miao, Boston University, Email: miaoj@bu.edu 
%           and Jieran Wu, Zhejiang University, Email: jw5ya@zju.edu.cn
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if min(eig(Sigma_pr))<1e-10
    disp('Sigular prior (smallest eigenvalue less than 1e-10): Stop')
    return
end

n = size(Sigma,1);

if norm(Sigma-Sigma_pr,'fro')<1e-6
    disp('No signal is acquired')
    C = 0; V = inf; r=0;
    return
end


SNR = pinv(Sigma) - pinv(Sigma_pr); %Signal to noise ratio 
[u,S,v] = svd(SNR);

% Optimal Signal Dimension
r = 0; 

for i = 1:n
    if S(i,i)/S(1,1) > 1e-3  % Condition number of precision matrix S cannot be too large      
        r = r+1;
    end
end

V = inv(S(1:r,1:r)); 

Delta = [eye(r,r), zeros(r,n-r)];

C = Delta*u';
    
end

