%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code solve for the steady state of the multivariate RI problem in the LQG framework
% Apply the modified value function iteration method using envelope condition
% The code works under the assumption A*A'+W is invertible
% 
% You need to install the software CVX that can be freely 
% downloaded from the website: http://cvxr.com/cvx/
% 
%% Input:
% A:       State transition matrix
% W:       State noise covariance matrix
% Omega:   One-period marginal benefit of information
% lambda:  Information cost parameter
% Lambda0: Initial guess of the Lagrange multiplier for the no forgetting constraints
% Sigma0:  Initial guess for the posterior covariance matrix of state
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Ouput:
% Sigma1:   Steady state posterior covariance matrix of state
% Sigma_pr: Steady state prior covariance matrix of state
% Theta1:   Steady state dynamic marginal benefit of information
% Lambda1:  Steady state Lagrange multiplier for the no-forgetting constraint 
%
% @ Written by Jianjun Miao, Boston University, Email: miaoj@bu.edu 
%           and Jieran Wu, Zhejiang University, Email: jw5ya@zju.edu.cn
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [Sigma1,Sigma_pr, Theta1, Lambda1] = RI_SS_EC(A,W,Omega,lambda,beta, Sigma0, Lambda0,varargin)
   
    if min(eig(A*A'+W))<1e-8
    disp('Invertibility Condition is violated: Stop')
    return
    end

% only want 2 optional inputs at most
numvarargs = length(varargin);
if numvarargs > 3
    error('myfuns:somefun2Alt:TooManyInputs', ...
        'requires at most 3 optional inputs');
end

opt = {1e-6 1 1}; %Default optional input value
opt(1:numvarargs) = varargin;
[tol a output] = opt{:}; % Error tolerance, updating weight, and screen display 

n=size(A,1); % State dimension

    if min(eig(W)) > 1e-8 %If W is invertible
     
        disp('W is invertible')

        err = 1; % initial iteration error
        iter = 0; % iteration count
        
        while err > tol & iter < 300
      
        cvx_begin sdp quiet
        variable Pi(n,n) semidefinite
        variable Sigma1(n,n) semidefinite 
        dual variable Lambda1
        minimize( trace(Omega*Sigma1)-0.5*lambda*(1-beta)*log_det(Sigma1)...
            +0.5*lambda*beta*(log_det(W)-log_det(Pi))-beta*trace(Lambda0*A*Sigma1*A'))  %% last term replace the continuation value function
        subject to
            Lambda1: A*Sigma0*A'+W>=Sigma1;
            [Sigma1-Pi, Sigma1*A';
            A*Sigma1, A*Sigma1*A'+W]>=0;
           %   log_det(Sigma1)>=-0.01; % Add additional entropy constraint
        cvx_end
 
       err = norm(Sigma0-Sigma1,'fro')/n;
       Lambda0 = a*Lambda1+(1-a)*Lambda0; % Update
       Sigma0  = a*Sigma1 + (1-a)*Sigma0; % Update
        
        iter = iter + 1;               
         Text = ['MVFI_EC_SS Iteration: ' num2str(iter) ', Error: ' num2str(err)];
         if output == 1
          disp(Text);  
         end
        end
            
    elseif min(abs(eig(A)))>1e-8 %If A is invertible, but W is not
    
        disp('A is invertible, but W is not')
        
        [U,D,V]= svd(W); 
         M = U*sqrt(D)*U'; %decompose W=M*M'
         
        err = 1; % initial iteration error
        iter = 0; % iteration count
        while err > tol & iter < 200
       
        cvx_begin sdp quiet
        variable Psi(n,n) semidefinite
        variable Sigma1(n,n) semidefinite
        dual variable Lambda1
        minimize(trace(Sigma1*Omega) -0.5*lambda*(1-beta)*log_det(Sigma1)...
             -0.5*lambda*beta*log_det(Psi)...
            -beta*trace(Lambda0*A*Sigma1*A')) 
        subject to
    Lambda1: A*Sigma0*A'+W>=Sigma1;     
            [eye(n)-Psi, M';
            M, A*Sigma1*A'+W]>=0;
        cvx_end
        
         err = norm(Sigma0-Sigma1,'fro')/n;
        Lambda0 = a*Lambda1+(1-a)*Lambda0; % Update
        Sigma0  = a*Sigma1 + (1-a)*Sigma0; % Update
        
        iter = iter + 1;               
        Text = ['MVFI_EC_SS Iteration: ' num2str(iter) ', Error: ' num2str(err)];
            if output == 1
                disp(Text);  
            end       
        end
          
    else
        
        disp('Both A and W are singular, but A*A^p+W is invertible')
        disp('Use CCP algorithm') 
       
        
        Sigmak = eye(n); %Initial point for linear approximation
        error2 = 1; % Initial outside iteration error
        iter2 = 0;  % Outside iteration count   
        tol = 1e-3;
        while error2 > tol & iter2 < 1000
        error1 = 1; % initial inside iteration error 
        iter1 = 0; % Inside iteration count
          
            while error1 > 1e-4 & iter1 < 1000
            cvx_begin sdp quiet
            variable Sigma1(n,n) semidefinite 
            dual variable Lambda1
            minimize ( trace(Omega*Sigma1)-0.5*lambda*log_det(Sigma1)...
            +0.5*lambda*beta*(log_det(A*Sigmak*A'+W)+trace( A'/(A*Sigmak*A'+W)*A*(Sigma1-Sigmak)))...%%Linear approximation to logdet(A*Sigma*A'+W)  
            -beta*trace(Lambda0*A*Sigma1*A')) %% last term replace the continuation value function
            subject to
            Lambda1: A*Sigma0*A'+W>=Sigma1;
            cvx_end
 
             error1 = norm(Sigmak-Sigma1,'fro')/norm(Sigmak);
           
             Sigmak = Sigma1;
        
             iter1 = iter1 + 1;               
            % Text = ['CCP Inside Iteration: ' num2str(iter1) ', Error: ' num2str(error1)];
            % disp(Text); 
            end
            
           error2 = norm(Sigma0-Sigma1,'fro')/norm(Sigma0);
           Lambda0 = a*Lambda1+(1-a)*Lambda0; % Update
           Sigma0  = a*Sigma1 + (1-a)*Sigma0; % Update
           iter2 = iter2 + 1;               
           Text = ['MVFI_EC_SS Iteration (CCP): ' num2str(iter2) ', Error: ' num2str(error2)];
           if output == 1
            disp(Text);  
            end     
        end
         
    end
    
    if output == 0
          disp(Text);  
    end
    Theta1 = lambda/2*inv(Sigma1) - Lambda1; 
    Sigma_pr = A*Sigma1*A'+W;
end

