%% This code solves for the steady state optimal information structure of the dynamic 
%% multivariate rational inattention problem in the LQG framework.
%% This code can handle both pure tracking problems and control problems
%% The code works under the assumption A*A'+W is invertible
%% Input:
% lambda     : Cost of information
% beta	     : Discount factor
% A 	     : State transition matrix
% W 	     : State covariance matrix
% Omega      : One period benefit matrix of information
% Theta0     : Initial guess for the dynamic benefit matrix of information
% Sigma_pr0  : Initial guess for the prior covariance matrix of state 
%% Output:
% Sigma     : Steady-state posterior covariance matrix of state
% Sigma_pr  : Steady-state prior covariance matrix of state
% Theta     : Steady-state dynamic marginal benefit matrix of information
% Lambda    : Lagrange multiplier matrix on the no-forgetting constriant -- Lambda*(Sigma-Sigma_p) = 0 
%
% @Written by Jianjun Miao, Boston University, email: miaoj@bu.edu
% 
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Sigma, Sigma_pr, Theta, Lambda, err, iter] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0,varargin) 

if min(eig(A*A'+W)) < 1e-8
        disp('A*A^p+W is not invertible.')
        return
end
    
% only want 2 optional inputs at most
numvarargs = length(varargin);
if numvarargs > 2
    error('myfuns:somefun2Alt:TooManyInputs', ...
        'requires at most 3 optional inputs');
end

opt = {1e-8 0.5}; %Default optional input value
opt(1:numvarargs) = varargin;
[tol a] = opt{:}; % Error tolerance and updating weight        
  

n   = size(A,1);  %Number of state variables
I   = eye(n,n);

    err     = 1;
    iter    = 0; 
    
	SqrtSigma_pr    = sqrtm(Sigma_pr0);
    
    while err > tol & iter <10^5
		if  min(eig(Theta0)) > -1e-10  %check if psd
            [U,D,v] = svd(SqrtSigma_pr*Theta0*SqrtSigma_pr); % Singular value decomposition
         else
            [U,D] = eig(SqrtSigma_pr*Theta0*SqrtSigma_pr); % Eigendecomposition  
        end
         D = diag((abs(diag(D))>1e-10).*diag(D)) ;
        U = (abs(U)>1e-10).*U ;

         Sigma     = SqrtSigma_pr*U/(max(2*D/lambda,I))*U'*SqrtSigma_pr; %Optimal posterior covariance matrix
    
        Sigma_pr   = A*Sigma*A' + W; % New prior covariance matrix
        
        err1    = norm(Sigma_pr - Sigma_pr0,'fro')/norm(Sigma_pr0,'fro'); %Check convergence error

        Sigma_pr0  = a*Sigma_pr + (1-a)*Sigma_pr0 ; %Update prior covariance matrix
        
        SqrtSigma_pr  = sqrtm(Sigma_pr0); %Squre root of updated prior covariance matrix  
        
        Theta = Omega + beta*A'/SqrtSigma_pr*U*min(D,0.5*lambda*I)*U'/SqrtSigma_pr*A; %Updated dynamic benefit matrix
        
        err2 = norm(Theta - Theta0,'fro')/norm(Theta0,'fro');
       
        err = err1 + err2;
       
        Theta0 = a*Theta+(1-a)*Theta0; %Update dynamic benefit matrix Theta
      
        iter  = iter + 1;
    end
    
    invSqrSigma_pr = pinv(SqrtSigma_pr); 
    Lambda = invSqrSigma_pr*U*max(lambda/2*I - D,0)*U'*invSqrSigma_pr; %Lagrange multiplier
     
    Text = ['FOC_SS Iteration: ' num2str(iter) ', Error: ' num2str(err)];
         disp(Text); 
end