%% This Matlab code computes the steady state of the multivariate RI problem in the LQG control framework
%% Use the MVFI method based on a sequence of static RI problems
%% The program works under the assumption that A*A'+W is invertible
% 
% You need to install the code armafit.m
% You need to install the software CVX for algorithm 1 that can be freely 
% downloaded from the website:
% http://cvxr.com/cvx/
% 
%% Input:
% A:          State transition matrix
% W:          State noise covariance matrix
% Omega:      One-period benefit matrix of information       
% lambda:     Information cost parameter
% beta:       Discount factor 
% Sigma_pr0:  Initial guess of prior covariance matrix of state
% Theta0:     Initial guess of dynamic marginal benefit matrix of information
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Ouput:
% Sigma1:    Steady-state posterior covariance matrix of state
% Sigma_pr1: Steady-state prior covariance matrix of state
% Theta1:    Steady-state dynamic marginal benefit matrix of information
% Lambda:    Steady-state Lagrange multiplier for the no-forgetting constraint
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by Jianjun Miao, Department of Economics, Boston University
% Emai: miaoj@bu.edu
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [Sigma1,Sigma_pr1,Theta1, Lambda] = RI_SS_SQ(A,W,Omega,lambda, beta,Sigma_pr0,Theta0,varargin)
    
    if min(eig(A*A'+W)) < 1e-8
        disp('A*A^p+W is not invertible.')
        return
    end

    % only want 2 optional inputs at most
numvarargs = length(varargin);
if numvarargs > 3
    error('myfuns:somefun2Alt:TooManyInputs', ...
        'requires at most 3 optional inputs');
end

opt = {1e-5 1 1}; %Default optional input value
opt(1:numvarargs) = varargin;
[tol a output] = opt{:}; % Error tolerance, updating weight, and screen display   
                    %If output ==1, more screen display; ==0, less screen display
n=size(A,1);

    iter =0;
    err = 1;

    while err >tol & iter< 600

    [Sigma1,Lambda] = RI_Static(lambda,Theta0,Sigma_pr0); %Compute the optimal posterior covariance matrix from a static RI model
   
    Sigma_pr1 = A*Sigma1*A'+W; % Prior

    err1  = norm(Sigma_pr1-Sigma_pr0,'fro')/norm(Sigma_pr0,'fro'); 

    Sigma_pr0 = a*Sigma_pr1+(1-a)*Sigma_pr0;  %Updating prior covariance matrix
   
    %Theta1 = Omega + 0.5*beta*lambda*A'*inv(Sigma_pr1)*A - beta*A'*Lambda*A; 
    Theta1 = Omega + 0.5*beta*lambda*A'*(Sigma_pr1\A) - beta*A'*Lambda*A;  %Aovid matrix inverse
    err2  = norm(Theta1-Theta0,'fro')/norm(Theta0,'fro');

    Theta0 = a*Theta1+(1-a)*Theta0; %Updating dynamic benefit matrix

        err  = err1+err2;
         
         iter = iter + 1;
         
         Text = ['MVFI_SQ_SS Iteration: ' num2str(iter) ', Error: ' num2str(err)];
         if output == 1
         disp(Text);
         end
    end
  if output == 0
      disp(Text)
  end
end




