%% This script solves for the steady state and transition dynamics for the pricing example in
%% Sims's hanndbook chapter using the modified value function iteration
%% method using envelope condition
% The code works under the assumption A*A'+W is invertible
%
% You need to install the software CVX that can be freely 
% downloaded from the website: http://cvxr.com/cvx/
%
%  @Written by Jianjun Miao Email: miaoj@bu.edu
% Department of Economics, Boston University
%
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
close all
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input Model Parameter values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

beta = 0.9;    % Discount factor

%%Sims's VAR(1) example
rho_1 = 0.95; %more persistent state
rho_2 = 0.40;
Q =[sqrt(1-rho_1^2) 0; 0 sqrt(1-rho_2^2)];
W = Q*Q'; %State noise variance
A = [rho_1,0; %State transition matrix
    0,rho_2];
G = [1,1];

%%MA case
%theta_1 = 0.8;
%theta_2 = 0.5;
%sigma = 0.5;
%W = [sigma^2, sigma^2,0;sigma^2,sigma^2,0;0,0,0];
%Q = [sigma, 0,0;sigma,0,0;0,0,0]; % W = Q*Q'; Afrouzi and Young approach
%A =[0, theta_1,theta_2
%    0, 0, 0
%    0, 1, 0];
%G = [1,0,0];

Omega = G'*G; %Weighting matrix
lambda = 2; %0.5;
if min(eig(A*A'+W))<1e-8
    disp('Invertibility Condition is violated: Stop')
    return
end
    
cvx_solver mosek %Select solver, sedumi, SDPT3 is the default

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Solve for the steady state
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sigma_pr0 = A*A'+ W;
Theta0 = Omega;
tic
% Use FOC based method or any method 
[SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0); 
SigmaSS
timeSSFOC = toc


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Transition dynamics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic


T = 20; % Transition period
n=size(A,1);
Sigma = zeros(n,n,T); % Sequence of posterior covariance matrices
Sigma_pr = zeros(n,n,T); % Sequence of prior covariance matrices
Lambda = zeros(n,n,T+1); % Sequence of Lagrange multipliers for the no-forgeting constraints


%Initial guess for the optimal posterior covariance matrix
for t=1:T
    Sigma(:,:,t) = SigmaSS; 
end

Lambda(:,:,T+1) = LambdaSS; %Terminal Lagrange multiplier

Sigma_pr0= 0.5*W; %0.5*(A*A'+W); %Initial prior covariance matrix

a = 0.6; %updating weight
tol = 1e-4;
err = 1;
iter = 0;
Sigmak = SigmaSS; %Initial guess for CCP

while err > tol & iter < 100
    for t=T:-1:2   
    Sigma_pr(:,:,t-1) = A*Sigma(:,:,t-1)*A'+W; %Given a sequence of posterior, construct the prior
    [Lambda(:,:,t), Sigma1(:,:,t)] = Jt(Lambda(:,:,t+1), Sigma_pr(:,:,t-1),Sigmak, A,W, Omega, lambda, beta);
    Sigmak = Sigma1(:,:,t); % Use previous solution as initial guess for CCP when both A and W are invertible
    end
    %initial time solution
    [Lambda(:,:,1),Sigma1(:,:,1),text1] = Jt(Lambda(:,:,2),Sigma_pr0,Sigmak, A, W, Omega, lambda, beta);
     
    for t=1:T
       Diff(t)  = norm(Sigma1(:,:,t)-Sigma(:,:,t),'fro')/norm(Sigma(:,:,t),'fro');
    end
    err = max(Diff);
   
    Sigma = a*Sigma1+(1-a)*Sigma; %Update the sequence of posteriors
         
         iter = iter + 1; 
         disp(text1)        
         Text = ['TD_EC Iteration: ' num2str(iter) ', Error: ' num2str(err)];
         disp(Text); 
                 
end
Sigma
toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function called in the main code
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Lambda1, Sigma1,text] = Jt(Lambda0, Sigma_pr0,Sigmak, A,W,Omega, lambda, beta)

n=size(A,1);

 if min(eig(W))>1e-8    
       text = 'W is invertible';
    
      cvx_begin sdp quiet
        variable Pi(n,n) semidefinite
        variable Sigma1(n,n) semidefinite 
        dual variable Lambda1
  
           minimize( trace(Omega*Sigma1)-0.5*lambda*(1-beta)*log_det(Sigma1)...
                -0.5*lambda*beta*log_det(Pi)-beta*trace(A'*Lambda0*A*Sigma1)) %remove constant log_det(W)
            %+0.5*lambda*beta*(log_det(W)-log_det(Pi))-beta*trace(A'*Lambda0*A*Sigma1))  %% last term replace the continuation value function
        subject to
        Lambda1:Sigma_pr0 >= Sigma1;
              [Sigma1-Pi, Sigma1*A';
               A*Sigma1, A*Sigma1*A'+W]>=0;
       cvx_end;
 
  elseif min(abs(eig(A)))>1e-8
       text = 'A is invertible';
       [U,D,V]= svd(W); 
       M = U*sqrt(D)*U'; %decompose W=M*M'
    cvx_begin sdp quiet       
        variable Pi(n,n) semidefinite
        variable Sigma1(n,n) semidefinite 
        dual variable Lambda1
        minimize(trace(Omega*Sigma1)-0.5*lambda*(1-beta)*log_det(Sigma1) -0.5*lambda*beta*log_det(Pi)...
           -beta*trace(A'*Lambda0*A*Sigma1)) %We removed the constant of log|A|
        subject to
        Lambda1: Sigma_pr0>=Sigma1;       
                [eye(n)-Pi, M';
                M, A*Sigma1*A'+W]>=0;
    cvx_end
   
   
   elseif min(eig(A*A'+W))>1e-8
       text =['Both A and W are singular, but A*A^p+W is invertible: ','  USe CCP algorithm'];
        
        err = 1; % initial iteration error
        tol = 1e-6; % convergence criteria
        iter = 0; % iteration count
       
        while err > tol & iter < 100
  
        cvx_begin sdp quiet
        variable Sigma1(n,n) semidefinite 
        dual variable Lambda1
            minimize ( trace(Omega*Sigma1)-0.5*lambda*log_det(Sigma1)...
            +0.5*lambda*beta*(log_det(A*Sigmak*A'+W)+trace( A'*inv(A*Sigmak*A'+W)*A*(Sigma1-Sigmak))))  %Linear approximation to logdet(A*Sigma*A'+W)
            subject to
            Lambda1: Sigma_pr0>=Sigma1;
        cvx_end
 
        err = norm(Sigmak-Sigma1,'fro')/n;
        Sigmak = Sigma1;    
          iter = iter + 1;                  
        end
    Text = ['GR CCP Iteration: ' num2str(iter) ', Error: ' num2str(err)];
          disp(Text); 
          
   end
       

end

