%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This Matlab program solves for the transition dynamics for the dynamic multivariate RI problem in the LQG framework 
% The code works under the assumption A*A'+W is invertible
%% INPUT:
% lambda:    Information cost parameter
% beta:      Discount factor
% Omega:     One-period marginal benefit of information
% A:         State transition equation
% W:         State innovation covariance matrix
% Sigma0:    Initial guess of the optimal positive semidefinite covariance matrix, n by n 
% Theta:     Terminal dynamic marginal benefit of information, nu by n matrix
% Sigma_pr0: Initial given positive semidefinite prior covariance matrix 
%
%% OUPUT:
% Sigma:    Sequence of optimal posterior covariance matrices, n by n by T array
% Sigma_pr: Sequence of optimal prior covariance matrices, n by n by T array
% Theta:    Sequence of dynamic benefit matrix, n by n by T array
% Lambda:   Sequence of Lagrange multipliers for the no-forgetting constraints, n by n by T array
%
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by Jianjun Miao, Department of Economics, Boston University
% Emai: miaoj@bu.edu
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
function [Sigma, Sigma_pr, Theta, Lambda] = RI_TD_FOC(lambda,beta,Omega, A, W, Sigma0, ThetaT,Sigma_pr0,T,varargin)

 if min(eig(A*A'+W)) < 1e-8
        disp('A*A^p+W is not invertible.')
        return
    end

    % only want 2 optional inputs at most
numvarargs = length(varargin);
if numvarargs > 2
    error('myfuns:somefun2Alt:TooManyInputs', ...
        'requires at most 3 optional inputs');
end

opt = {1e-7 1}; %Default optional input value
opt(1:numvarargs) = varargin;
[tol a] = opt{:}; % Error tolerance and updating weight

n = size(A,1); % State dimension
I= eye(n);          % Identity matrix
Sigma = zeros(n,n,T-1); % Sequence of posterior covariance matrices
Theta = zeros(n,n,T);   % Sequence of dynamic marginal benefits
Sigma_pr = zeros(n,n,T); % Sequence of prior covariance matrices
U = zeros(n,n,T); % Sequence of eigenvector matrices  
D = zeros(n,n,T); % Sequence of eigenvalue matrices
Lambda = zeros(n,n,T); % Sequence of Lagrange multipliers for the no-forgeting constraints

Sigma_pr(:,:,1) = Sigma_pr0;  %Fixed initial prior covariance
Theta(:,:,T) = ThetaT;    % Terminal condition for the dynamic marginal benefit

    for t=1:T-1;
        Sigma(:,:,t) = Sigma0; %initial guess for the sequence of posterior covariance matrices
    end
    
    err1 = 1;
    iter = 0;
    %Main loop
    while err1 > tol & iter < 10^4;
    
        for t=2:T; %Compute prior
            Sigma_pr(:,:,t) = A*Sigma(:,:,t-1)*A'+W;
        end;
    
        for t=1:T;  %Compute square root of prior covariance
            SqrSigma_pr(:,:,t) = sqrtm(Sigma_pr(:,:,t));
        end

        for j = T:-1:2;  % Solve backward for the  dynamic marginal benefit
            
            DTheta(:,:,j) = SqrSigma_pr(:,:,j)*Theta(:,:,j)*SqrSigma_pr(:,:,j);  %matrix for eigendecomposition  
            if min(eig(Theta(:,:,j))) >-1e-10  %check if psd
            [U(:,:,j),D(:,:,j),v] = svd(DTheta(:,:,j)); %Sigular value decomposition
            else
            [U(:,:,j),D(:,:,j)] = eig(DTheta(:,:,j)); %Eigendecomposition
            end
            Lambda(:,:,j)  = inv(SqrSigma_pr(:,:,j))*U(:,:,j)*max(lambda*I/2 - D(:,:,j),0)*U(:,:,j)'*inv(SqrSigma_pr(:,:,j));
            Theta(:,:,j-1) = Omega + beta*A'*(lambda/2*inv(Sigma_pr(:,:,j))- Lambda(:,:,j))*A;
        end
   
    %Initial values
        DTheta(:,:,1) = SqrSigma_pr(:,:,1)*Theta(:,:,1)*SqrSigma_pr(:,:,1);  %matrix for eigendecomposition  
          if min(eig(Theta(:,:,1))) >-1e-10  %check if psd
            [U(:,:,1),D(:,:,1),v] = svd(DTheta(:,:,1)) ;
            else
            [U(:,:,1),D(:,:,1)] = eig(DTheta(:,:,1)) ;
          end
          
         Lambda(:,:,1)   = inv(SqrSigma_pr(:,:,1))*U(:,:,1)*max(lambda*I/2 - D(:,:,1),0)*U(:,:,1)'*inv(SqrSigma_pr(:,:,1));
        
        for t=1:T-1; % Solve forward for updated posterior covariance matrix
          Sigma_new(:,:,t) = lambda/2*SqrSigma_pr(:,:,t)*U(:,:,t)/max(D(:,:,t),lambda*I/2)*U(:,:,t)'*SqrSigma_pr(:,:,t);
        end
    
        for t=1:T-1;
            err(t) = norm (Sigma(:,:,t)-Sigma_new(:,:,t),'fro');
        end    
        err1 = max(err)/n;
    
    Sigma = a*Sigma_new+(1-a)*Sigma; %Updating sequence of posterior covariance matrices
    iter=iter+1;
    end
    Text = ['FOC_TD Iteration: ' num2str(iter) ', Error: ' num2str(err1)];
    disp(Text); 

end

