%% This Matlab code computes the transition dynamics of the multivariate RI problem in the LQG control framework
%% Use the MVFI method based on a sequence of static RI problems.
%% The program works under the assumption that A*A'+W is invertible
% 
% You need to install the software CVX downloaded from the website:
% http://cvxr.com/cvx/
%
%% Input:
% A:          State transition matrix
% W:          State noise covariance matrix
% Omega:      One-period benefit of information       
% lambda:     Information cost parameter
% beta:       Discount factor 
% Sigma_pr0:  Initial prior covariance matrix
% Sigma_prT:  Terminal prior covariance matrix
% LambdaT:    Terminal Lagrange multiplier for the no forgetting constraint
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Ouput:
% Sigma1:    Sequence of posterior covariance matrices for the state
% Sigma_pr1: Sequence of prior covariance matrices for the state
% Theta:     Sequence of dynamic benefit matrix of information
% Lambda:    Sequence of Lagrange multipliers for the no-forgetting constraint
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by Jianjun Miao, Department of Economics, Boston University
% Emai: miaoj@bu.edu
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [Sigma1, Sigma_pr1, Theta, Lambda] = RI_TD_SQ(A, W,Omega, lambda,beta, LambdaT, Sigma_prT, Sigma_pr0,T,varargin);

  if min(eig(A*A'+W)) < 1e-8
        disp('A*A^p+W is not invertible.')
        return
    end

    % only want 2 optional inputs at most
numvarargs = length(varargin);
if numvarargs > 2
    error('myfuns:somefun2Alt:TooManyInputs', ...
        'requires at most 3 optional inputs');
end

opt = {1e-5 0.5}; %Default optional input value
opt(1:numvarargs) = varargin;
[tol a] = opt{:}; % Error tolerence and updating weight

n=size(A,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Sigma_pr = zeros(n,n,T+1); % Sequence of posterior covariance matrices
Sigma_pr = zeros(n,n,T+1); % Sequence of prior covariance matrices
Lambda =  zeros(n,n,T+1); % Sequence of Lagrange multipliers for the no-forgeting constraints
Theta =  zeros(n,n,T+1);
Sigma1 = zeros(n,n,T);

%Initial guess for the optimal sequence of posterior covariance matrices
for t=1:T+1
    Sigma_pr(:,:,t) = Sigma_prT; 
end

Lambda(:,:,T+1) = LambdaT; %Terminal Lagrange multiplier
Sigma_pr(:,:,1) = Sigma_pr0; 

iter =0;
err = 1;

while err >tol & iter< 100

    for t=T:-1:1 ; %Solve Theta backward       
        %Theta(:,:,t) = Omega + beta*lambda/2*A'*inv(Sigma_pr(:,:,t+1))*A - beta*A'*Lambda(:,:,t+1)*A;
        Theta(:,:,t) = Omega + beta*lambda/2*A'*(Sigma_pr(:,:,t+1)\A) - beta*A'*Lambda(:,:,t+1)*A; %Avoid inverse
        [Sigma1(:,:,t),Lambda(:,:,t)] = RI_Static(lambda,Theta(:,:,t),Sigma_pr(:,:,t)); %Compute optimal posterior covariance matrix from a static RI problem  
        
    end
    
    for t=1:T-1; %Solve prior covariance matrix forward
        Sigma_pr1(:,:,t+1) = A*Sigma1(:,:,t)*A'+W; %Update      
    end
    
    Sigma_pr1(:,:,1) = Sigma_pr(:,:,1); %Fix the initial prior covariance matrix at the exogenous value
     
  
    for t=2:T;   
        err1(t)  = norm(Sigma_pr1(:,:,t)-Sigma_pr(:,:,t),'fro')/n;
    end
 
    err = max(err1);
 
 Sigma_pr(:,:,1:T) = a*Sigma_pr1(:,:,1:T)+(1-a)*Sigma_pr(:,:,1:T);  % Update prior covariance matrix
 
         iter = iter + 1; 
                 
         Text = ['MVF_SQ_TD Iteration: ' num2str(iter) ', Error: ' num2str(err)];
         disp(Text); 
         
end

end






