%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This Matlab program solves for the transition dynamics and steady state 
% of the posterior covariance matrix 
% for the tracking problem in Sims Handbook example, which is also in
% Section 4 of Miao, Wu, and Young (2020). 
% The code applies under the assumption that W is invertible. One can suitability modify the code to incorporate the
% case in which A is invertible or A*A'+W is invertible.
%
% You need to install the software CVX that can be freely 
% downloaded from the website:
% http://cvxr.com/cvx/
%
% @ Written by Jianjun Miao. Email: miaoj@bu.edu
% Department of Economics, Boston University
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all; close all;

global A W Omega lambda beta T n

tic
%Generate parameter data
rho_1 = 0.95; %more persistent state
rho_2 = 0.40; 
beta = 0.9;  %Discount factor

W = [0.0975 0;   %State noise covariance matrix
     0 0.84];

A = [rho_1,0; %State transition matrix
    0,rho_2];

G = [1,1];

Omega = G'*G; %Weighting matrix
lambda = 2; % Information cost. My lambda corresponds to 2*lambda in Sims handbook example

n = size(A,1); %State dimension

tol = 1e-6; % Error tolerance for optimal posterior covariance matrix

SS = 1; % Indicator variable, equal to 1 if solve for steady state; equal to 0 if solve for the transition dynamics

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cvx_solver mosek %sedumi


T = 100; % Maximal number of value function iterations

Sigma = zeros(n,n,T); % Initialize sequence of optimal posterior covariance matrices

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute infinite-horizon value function by value function iteration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Find value function by VFI')
Sigma0 = 0.5*W; %A*A'+W; %Can use any positive definite matrix 
z(T)=JT(Sigma0); %Start with the last period

for t=T-1:-1:1;
    z(t) = Jt(Sigma0,t);
    error =abs(z(t+1)-z(t))/z(t);
    Text = ['Iteration ' num2str(T-t) ', Value function error: ' num2str(error)];
    disp(Text);
    if error < 0.005 
     disp(['Number of iterations until value function convergence = ', num2str(T-t)])
        break
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the optimal sequence of covariance matrices
% Jt(Sigma,t) is the infinite-horizon value function computed above.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Compute time zero optimal posterior covariance matrix

if SS == 0;
    disp('Solve for transition dynamics starting from any prior')
    
    Sigma0 = 0.5*W; %A*A'+W; %Starting prior covariance matrix
    
    cvx_begin sdp quiet
    variable Pi(n,n) semidefinite
    variable Sigma1(n,n) semidefinite  %Time 0 optimal posterior covariance matrix
    minimize( trace(Omega*Sigma1)-0.5*lambda*(1-beta)*log_det(Sigma1)...
      +0.5*lambda*beta*(log_det(W)-log_det(Pi))+beta*Jt(Sigma1,t)) 
    subject to
        Sigma0 >= Sigma1;
        [Sigma1-Pi, Sigma1*A';
            A*Sigma1, A*Sigma1*A'+W]>=0;
    cvx_end

    Sigma(:,:,1) = Sigma0; %Initial prior    
    
else 
     % Use golden rule solution as the initial guess to solve steady state
     % About one minute faster
     disp('Solve for steady state starting at golden rule prior')
     
    [SigmaGR,Sigma_prGR, LambdaGR,ThetaGR] = RI_GR(A,W,Omega,0,lambda,1);  

     Sigma1 = SigmaGR; % Starting at the golden rule solution speeds up convergence to steady state     
end

% Compute the sequence of optimal posterior covariance matrices starting from time 1 on

i=1;
while i<T;
    [z2,Sigma2] = Jt(Sigma1,t); %Compute policy function, Sigma_t given Sigma_{t-1} 
    error = norm(Sigma1-Sigma2,'fro')/norm(Sigma1,'fro');
    Text = ['Period t = ' num2str(i) ', Posterior Covariance error: ' num2str(error)];
    disp(Text)
    if error < tol  %check convergence of Sigma_t
      disp(['Convergence in period =', num2str(i)]);
        break    
    end
    i=i+1;
    Sigma(:,:,i)= Sigma1;
    Sigma1 = Sigma2; %Update Sigma_t recursively over time
        
end


disp('Sequence of posterior covariance matrices')
Sigma(:,:,1:i)

toc


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions called in the main prograqm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y = Jtp1(X,varargin)
% Value function in period t+1
global T
if T==varargin{1}; %Last period starting of iteration
    y=JT(X);
else
t=varargin{1};
[y,z]=Jt(X,t); %Updating
end
end

function [cvx_optval,Sigma1] = Jt(X,varargin)
%Value function in period t 
global A W Omega lambda beta n
t=varargin{1};
cvx_begin sdp quiet
 %cvx_solver sedumi
   variable Pi(n,n) semidefinite
   variable Sigma1(n,n) semidefinite  
   minimize( trace(Omega*Sigma1)-0.5*lambda*(1-beta)*log_det(Sigma1)...
      +0.5*lambda*beta*(log_det(W)-log_det(Pi))+beta*Jtp1(Sigma1,t+1)) 
 subject to
        A*X*A'+W>=Sigma1;
        [Sigma1-Pi, Sigma1*A';
            A*Sigma1, A*Sigma1*A'+W]>=0;
cvx_end
end


function cvx_optval = JT(X)
% Value function in the last period T
global A W Omega lambda n
cvx_begin sdp quiet
   variable Sigma1(n,n) semidefinite  
   minimize( trace(Omega*Sigma1)-0.5*lambda*log_det(Sigma1) )
   subject to
        A*X*A'+W>=Sigma1;
cvx_end
end


