%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This code solves for the steady state information structure for the pricing example in
%Sims (2011) using the modified value function iteration methods
% We introduce a new entropy constraint on the posterior covariance matrix.
% You need to install the software CVX that can be freely 
% downloaded from the website the software CVX from http://cvxr.com/cvx/.
%
%  @Written by Jianjun Miao Email: miaoj@bu.edu
% Department of Economics, Boston University
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all

cvx_solver mosek %Select sdpt3 or sudumi if mosek does not work
 
%% Generate parameter data

rho_1 = 0.95; %more persistent state
rho_2 = 0.40;
beta = 0.9;  %Discount factor

Q =[sqrt(1-rho_1^2) 0; 0 sqrt(1-rho_2^2)]; 

W = Q*Q'; %State noise variance matrix

A = [rho_1,0; %State transition matrix
    0,rho_2];

G = [1,1];

n = size(A,1);

Omega = G'*G; %Benefit matrix of information
lambda = 2; % Information cost. My lambda corresponds to 2*lambda in Sims handbook example

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Our preferred VFI method based on a sequence of static RI problems
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tstart = tic;
Theta0 = Omega; 
Sigma_pr0 = A*A'+W; %Initial guess must be feasible.

a =1; %Updating weight on the new iteration
tol=1e-6; %Error tolerance
iter =0;
    err = 1;

    while err >tol & iter< 10^3

    [Sigma1,Lambda] = RI_StaticN(lambda,Theta0,Sigma_pr0); %Compute the optimal posterior covariance matrix from a static RI model
   
    Sigma_pr1 = A*Sigma1*A'+W; % Prior

    err1  = norm(Sigma_pr1-Sigma_pr0,'fro')/n; 

    Sigma_pr0 = a*Sigma_pr1+(1-a)*Sigma_pr0;  %Updating prior covariance matrix
   
    Theta1 = Omega + 0.5*beta*lambda*A'*(Sigma_pr1\A) - beta*A'*Lambda*A; 
    err2  = norm(Theta1-Theta0,'fro')/n;

    Theta0 = a*Theta1+(1-a)*Theta0; %Updating dynamic benefit matrix

        err  = err1+err2;
         
         iter = iter + 1; 
                 
         Text = ['Iteration: ' num2str(iter) ', Error: ' num2str(err)];
         disp(Text); 
         
    end

tendSQ = toc(tstart); 

SigmaSQ = Sigma1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MVFI method using envelope condition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sigma0 = eye(n); %Initial guess must be feasible
Lambda0 = eye(n); 

tic
        error = 1; % initial iteration error
        iter = 0; % iteration count
      if eig(W)>1e-6
          disp('W is invertible')
        while error > tol & iter < 100
      
        cvx_begin sdp quiet
       % cvx_solver sedumi
        variable Pi(n,n) semidefinite
        variable Sigma1(n,n) semidefinite 
        dual variable Lambda1
        minimize( trace(Omega*Sigma1)-0.5*lambda*(1-beta)*log_det(Sigma1)...
            +0.5*lambda*beta*(log_det(W)-log_det(Pi))-beta*trace(Lambda0*A*Sigma1*A'))  %% last term replace the continuation value function
        subject to
            Lambda1: A*Sigma0*A'+W>=Sigma1;
             [Sigma1-Pi, Sigma1*A';
             A*Sigma1, A*Sigma1*A'+W]>=0;
            log_det(Sigma1)>=-0.01; % Add additional entropy constraint
        cvx_end
 
       error = norm(Sigma0-Sigma1,'fro')/n;
       Lambda0 = a*Lambda1+(1-a)*Lambda0; % Update
       Sigma0  = a*Sigma1 + (1-a)*Sigma0; % Update
        
        iter = iter + 1;               
         Text = ['Iteration: ' num2str(iter) ', Error: ' num2str(error)];
        disp(Text);        
        end
     end
      
tEC=toc;

SigmaEC = Sigma1;


disp(['Our MVFI based on sequence of static RI problems, Computation time: ' num2str(tendSQ)])
 SigmaSQ 
 
 disp(['Our MVFI method using envelope condition, Computation time: ' num2str(tEC)])
SigmaEC



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Sigma1,Lambda] = RI_StaticN(lambda,Omega,Sigma_pr) 
%Static RI model with an additional constraint
n=size(Sigma_pr,1); % State dimension
cvx_begin sdp quiet
   variable Sigma1(n,n) semidefinite
   dual variable Lambda
   minimize( trace(Omega*Sigma1)-0.5*lambda*log_det(Sigma1))  
 subject to
          Lambda: Sigma_pr>= Sigma1;  %No-forgetting constraint
          log_det(Sigma1)>=-0.01; % Add additional entropy constraint
 cvx_end
end

