
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code computes the steady state and transition dynamics of the optimal information structure
% for the pricing example in Sims (2011) using three methods:
% (i) our FOC based method, (ii) MVFI method based on a squence of static RI problems.
%
% You need to install the software CVX that can be freely 
% downloaded from the website:
% http://cvxr.com/cvx/
%
% Written by Jianjun Miao, Department of Economics, Boston University
% Emai: miaoj@bu.edu
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; close all;

cvx_solver mosek

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameter values
%
beta = 0.9;    % Discount factor
rho_1 = 0.95; %more persistent state
rho_2 = 0.40;
beta = 0.90;  %Discount factor
Q =[sqrt(1-rho_1^2) 0; 0 sqrt(1-rho_2^2)];
W = Q*Q'; %State noise variance
A = [rho_1,0; %State transition matrix
    0,rho_2];
G = [1,1];
Omega = G'*G; %Weighting matrix
lambda = 2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Solve for the steady state
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic
Sigma0 = A*A'+ W;
Theta0 = Omega;
% FOC approach to verify solution
[SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma0); 
timeSSFOC = toc;

% Initial conjecture of multiplier
T = 30;
n = size(A,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Forward-backward shooting method based on FOC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sigma0 = SigmaSS;
ThetaT = ThetaSS;
Sigma_pr0 = 0.5*W;
tic
[SigmaFOC, Sigma_prFOC, ThetaFOC, LambdaFOC] = RI_TD_FOC(lambda,beta,Omega, A, W, Sigma0, ThetaT,Sigma_pr0,T);
timeTDFOC = toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Forward-backward shooting method based on a sequence of static RI models
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sigma_prT = Sigma_prSS; %Terminal condition
LambdaT = LambdaSS;  %Terminal condition

tic
tol = 1e-5;
a = 1;
[SigmaSQ, Sigma_prSQ, ThetaSQ, LambdaSQ] = RI_TD_SQ(A, W,Omega, lambda,beta, LambdaT, Sigma_prT, Sigma_pr0,T, tol, a);
timeTDSQ = toc

disp('Check Solutions from the two methods')

SigmaSQ(:,:,1:3)
SigmaFOC(:,:,1:3)

SigmaSS;

disp('Time_SS_FOC, Time_TD_FOC, Time_TD_SQ')

[timeSSFOC, timeTDFOC, timeTDSQ]




