%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This Matlab program solves for the transition dynamics 
% of the posterior covariance matrix 
% for the tracking problem in Sims Handbook example, which is also in
% Section 4 of Miao, Wu, and Young (2020).
% Call Matlab functions RI_SS_FOC.m, RI_IRF and RI_TD_FOC.m
%
% @ Written by Jianjun Miao. Email: miaoj@bu.edu
% Department of Economics, Boston University
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


global beta

%Generate baseline parameter data
%beta = 0.9;    % Discount factor

rho1 = 0.95;  % more persistent state
rho2 = 0.40;  % less persistent state

sigma1 = sqrt(0.0975); 
%sigma1 = sqrt(1-rho1^2); % Choose innovation variance to keep stationary variance at 1
sigma2 = sqrt(1-rho2^2); %0.84;
%sigma2=sqrt(0.86);

W=[sigma1^2 0;   %State noise covariance matrix
    0 sigma2^2];
A = [rho1,0;   %State transition matrix
    0,rho2];
G = [1,1];
Omega = G'*G; %Weighting matrix
lambda = 4; % Information cost. My lambda corresponds to 2*lambda in Sims handbook example
%lambda = 2;
n = size(A,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Solve for the steady state
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sigma_pr0 = A*A'+W;
Theta0 = Omega;

tic
[SigmaSS, Sigma_pr, ThetaSS, Lambda] = RI_SS_FOC(lambda,beta,A,W, Omega, Theta0, Sigma_pr0);
toc

[C,V,r] = RI_SIG(SigmaSS,Sigma_pr,ThetaSS,lambda); %Signal dimension = rank(SNR)


%disp('rho1;  Signal weight vector;  Signal noise; Signal dimension')
[rho1 C, V r] 
%SigmaSS

B = zeros(2,1); %Exogenous state
F = -G; % policy function y = -F*x = G*x
period = 30;
[X, u,irfyf] = RI_IRF(A,B,C,F,SigmaSS,V,W,period);
%time = 1:period;
%plot(time-1,u(1,:,1),'--*', time-1,u(1,:,2),'-o');
irfy = u(1,:,1:2); %IRF to two structural shocks

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve for transition dynamics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

T = 50;  % Transition period

%Initialization

ThetaT = ThetaSS;  % Steady state marginal benefit matrix 
          
Sigma_pr0 =0.5*W; % Initial prior covariance matrix   

Sigma0 = SigmaSS; % Steady state posterior covariance matrix, beta=0.9
         
 tic    
[Sigma, Sigma_pr, Theta, Lambda] = RI_TD_FOC(lambda,beta,Omega, A, W, Sigma0, ThetaT,Sigma_pr0,T);
toc

 %Check if the sequence of posterior covariance matrices converges 
err2 = norm(Sigma(:,:,T-2)-Sigma(:,:,T-1));
Sigma(:,:,T-2);
Sigma0;

T=20;
Sigma1=zeros(1,T);
time=1:T;
dim = zeros(1,T);

for t=1:T;
        [C,V,dim(t)] = RI_SIG(Sigma(:,:,t),Sigma_pr(:,:,t),Theta(:,:,t),lambda);
        if dim(t)==0
            disp(['t = ',num2str(t-1)])
        elseif dim(t) ==1
            C1(:,:,t) = C;
            V1(:,:,t)=V;
        end
    Sigma1(t) = Sigma(1,1,t);
    Sigma2(t) = Sigma(2,2,t);
end


