%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This Matlab program solves for the transition dynamics 
% of the posterior covariance matrix 
% for the tracking problem in Sims Handbook example, which is also in
% Section 4 of Miao, Wu, and Young (2020).
% Call Matlab functions RI_SS_FOC.m, RI_SIG.m, and RI_TD_FOC.m
%
% @ Written by Jianjun Miao. Email: miaoj@bu.edu
% Department of Economics, Boston University
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all

%Generate baseline parameter data
beta = 0.9;    % Discount factor

rho1 = 0.95;  % more persistent state
rho2 = 0.40;  % less persistent state

sigma1 = sqrt(0.0975); 
%sigma1 = sqrt(1-rho1^2); % Choose innovation variance to keep stationary variance at 1
sigma2 = sqrt(1-rho2^2); %sqrt(0.84);
sigma2 = sqrt(0.84);
W = [sigma1^2 0;   %State noise covariance matrix
    0 sigma2^2];
A = [rho1,0;   %State transition matrix
    0,rho2];
G = [1,1];
Omega = G'*G; %Weighting matrix
lambda = 2; % Information cost. My lambda corresponds to 2*lambda in Sims handbook example

n = size(A,1); %State dimension
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Panel A: lambda = 2
disp('******************************************')
disp('Panel A: lambda =2')
disp('******************************************')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Solve for the steady state
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sigma_pr0 = W;
Theta0 = Omega;

[SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0); 

SigmaSS


%% Comparision with Afrouzi and Yang (2020) algorithm
%H =G';
%Q=[sigma1,0;0 sigma2];
%omega = 1; % Their omega is our lambda/2 
%[Sigma1,Sigma_p,Lambda,Omega0,Y,Sigma_z,K] = Solve_RI_Dynamics(omega,beta,A,Q,H,Theta0,Sigma0); 
%Sigma_p
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[C1,V,r]=RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda); %Signal structure

CN=C1/C1(2);
VN=V/C1(2)^2;

disp('rho1;  Steady-state signal weight vector;  Signal noise; Signal dimension')
[rho1 CN, VN r] 
SigmaSS


tic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve for transition dynamics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

T = 50;  % Transition period

%Initialization

ThetaT = ThetaSS;  % Steady state marginal benefit matrix 
          
Sigma_pr0 =0.5*W; % Initial prior covariance matrix   

Sigma0 = SigmaSS; % Steady state posterior covariance matrix, beta=0.9
         
     
[Sigma, Sigma_pr, Theta, Lambda] = RI_TD_FOC(lambda,beta,Omega, A, W, Sigma0, ThetaT,Sigma_pr0,T);

toc
 
 %Check if the sequence of posterior covariance matrices converges 
err2 = norm(Sigma(:,:,T-2)-Sigma(:,:,T-1));
Sigma(:,:,T-2);
Sigma0;

T=20;
time=1:T;
dim1 = zeros(1,T);
for t=1:T;
   [C,V,dim1(t)] = RI_SIG(Sigma(:,:,t),Sigma_pr(:,:,t),Theta(:,:,t),lambda);
        if dim1(t)==0
            disp(['t = ',num2str(t-1)])
        elseif dim1(t) ==1
            C1(:,:,t) = C;
            V1(:,:,t)=V;
        end
    Sigma1(t) = Sigma(1,1,t);
    Sigma2(t) = Sigma(2,2,t);
end


%% Panel A: lambda = 0.2;
disp('******************************************')
disp('Panel A: lambda = 0.2')
disp('******************************************')

lambda = 0.2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Solve for the steady state
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Sigma_pr0 = W;
Theta0 = Omega;

[SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma_pr0); 


[C1,V,r]=RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda); %Signal structure

CN=C1/C1(2);
VN=V/C1(2)^2;

disp('rho1;  Steady-state signal weight vector;  Signal noise; Signal dimension')
[rho1 CN, VN r] 
SigmaSS


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve for transition dynamics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

T = 50;  % Transition period

%Initialization

ThetaT = ThetaSS;  % Steady state marginal benefit matrix 
          
Sigma_pr0 =0.5*W; % Initial prior covariance matrix   

Sigma0 = SigmaSS; % Steady state posterior covariance matrix, beta=0.9
         
     
[Sigma, Sigma_pr, Theta, Lambda] = RI_TD_FOC(lambda,beta,Omega, A, W, Sigma0, ThetaT,Sigma_pr0,T);

 
 %Check if the sequence of posterior covariance matrices converges 
err2 = norm(Sigma(:,:,T-2)-Sigma(:,:,T-1));
Sigma(:,:,T-2);
Sigma0;

T=20;
time=1:T;
dim2 = zeros(1,T);
for t=1:T;
   [C,V,dim2(t)] = RI_SIG(Sigma(:,:,t),Sigma_pr(:,:,t),Theta(:,:,t),lambda);
        if dim2(t)==0
            disp(['t = ',num2str(t-1)])
        elseif dim2(t) ==1
            C1(:,:,t) = C;
            V1(:,:,t)=V;
        end
    Sigma1(t) = Sigma(1,1,t);
    Sigma2(t) = Sigma(2,2,t);
end


 subplot(1,2,1)
 plot(time-1,dim1,'-o',time-1,dim2,'-*') 
 xlabel('time')
 legend('\lambda=2','\lambda=0.2','Location','SouthEast')
title('A. Sigmal dimension')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Panel B: rho1=1, lambda = 4
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('******************************************')
disp('Panel B: rho1=1, lambda =4')
disp('******************************************')

rho1 = 1;
lambda = 4;

A = [rho1,0;   %State transition matrix
    0,rho2];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Solve for the steady state
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Sigma0 = W;
Theta0 = Omega;

[SigmaSS, Sigma_prSS, ThetaSS, LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma0); 


[C1,V,r] = RI_SIG(SigmaSS,Sigma_prSS,ThetaSS,lambda); %Signal dimension = rank(SNR)

CN=C1/C1(2);
VN=V/C1(2)^2;

disp('rho1;  Steady-state signal weight vector;  Signal noise; Signal dimension')
[rho1 CN, VN r] 
SigmaSS


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve for transition dynamics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

T = 50;  % Transition period

%Initialization

ThetaT = ThetaSS;  % Steady state marginal benefit matrix 
          
Sigma_pr0 =0.5*W; % Initial prior covariance matrix   

Sigma0 = SigmaSS; % Steady state posterior covariance matrix, beta=0.9
         
     
[Sigma, Sigma_pr, Theta, Lambda] = RI_TD_FOC(lambda,beta,Omega, A, W, Sigma0, ThetaT,Sigma_pr0,T);

 
 %Check if the sequence of posterior covariance matrices converges 
err2 = norm(Sigma(:,:,T-2)-Sigma(:,:,T-1));
Sigma(:,:,T-2);
Sigma0;


% Determine signal dimension over time
T=20;
time=1:T;
dim = zeros(1,T);

for t=1:T;
    [C,V,dim(t)] = RI_SIG(Sigma(:,:,t),Sigma_pr(:,:,t),Theta(:,:,t),lambda);
        if dim(t)==0
            disp(['t = ',num2str(t-1)])
        elseif dim(t) ==1
            C1(:,:,t) = C;
            V1(:,:,t)=V;
        end

    Sigma1(t) = Sigma(1,1,t);
    Sigma2(t) = Sigma(2,2,t);
end


 subplot(1,2,2)
 plot(time-1,dim,'-o',time-1,Sigma1,'-s',time-1,Sigma2,'-*') 
 title('B. A unit root')
 xlabel('time')
 legend('signal dim','\Sigma_t(1,1)','\Sigma_t(2,2)','Location','SouthEast')
 
 print -depsc FigNew1.eps
 