%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This Matlab program solves for the the steady state and transition dynamics 
% in the univariate example of Sims (2011). It verifies Proposition 7 in
% MWY (2020).
%
% @ Written by Jianjun Miao. Email: miaoj@bu.edu
% Department of Economics, Boston University
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic

clear all
close all

%Generate parameter data

rho  = 2;  % more persistent state
beta = 0.8;    % Discount factor

if abs(rho)<1
    W = 1-rho^2; %Uncondition variance is 1; 
else
    W = 0.1;
end

A = rho;

Omega = 1; %Weighting matrix
lambda = 3; % Information cost. My lambda corresponds to 2*lambda in Sims handbook example

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%Create and solve problem numerically
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%initial guess
Sigma0 = 0.1*W; %W
Theta0 = 1;
[SigmaSS, Sigma_prSS, ThetaSS,LambdaSS] = RI_SS_FOC(lambda,beta,A,W,Omega, Theta0,Sigma0);

disp('Numerical steady state posterior variance')
SigmaSS

T = 30;  % Transition period

%Initialization

ThetaT = ThetaSS;  %Steady state marginal benefit matrix 
          
Sigma_pr0 = 0.1*W;   %Initial prior


Sigma0 = SigmaSS;  %Steady state posterior covariance matrix, beta=0.9

%Numerical transition dynamics
[Sigmat, Sigma_prt, Thetat, Lambdat] = RI_TD_FOC(lambda,beta,Omega, A, W, Sigma0, ThetaT,Sigma_pr0,T);
     
 %Check if the sequence of posterior covariance matrices converges 
 
 %err2 = norm(Sigma(:,:,T-2)-Sigma(:,:,T-1))

%Sigma(:,:,T-2)
%Sigma0

T=20;
for t=1:T;
    Sigma1(t)=Sigmat(:,:,t);
end

time=1:T;
%plot(time,D,'-o',time,lambda/2*ones(1,T))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Use Proposition 7 to compute true solution
%Check SS True solution
sigmas = 1/(4*rho^2)*(-2*W+(1-beta)*lambda*rho^2+sqrt((2*W-(1-beta)*lambda*rho^2)^2+8*rho^2*lambda*W));
if abs(rho)<1
    disp('True steady state posterior variance for abs(rho)<1')
    sigmass = min(W/(1-rho^2),sigmas)
    disp('Unconditional stationry variance')
    Varx = W/(1-rho^2)
else
    disp('True steady state posterior variance for abs(rho)>=1 by Proposition 7')
    sigmas
end

disp('True steady-state signal noise variance')
V= sigmas*(rho^2*sigmas+W)/((rho^2-1)*sigmas+W)

%True transition dynamics using Proposition 7
Sigma_true(1) = min(Sigma_pr0,sigmas);
Sigma_pr =zeros(1,T);
for t=1:T-1;
    Sigma_true(t+1)=min(rho^2*Sigma_true(t)+W,sigmas);    
end

for t=2:T
Sigma_pr_true(t) = (rho^2*Sigma_true(t-1)+W);
end
Sigma_pr_true(1) = Sigma_pr0;
(1./Sigma_true-1./Sigma_pr_true).^(-1);

disp('The difference between the true and numerical solutions for sequence of posteriors')
norm(Sigma1-Sigma_true)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Plot transition dynamics

figure
plot(time,Sigma_true, time,Sigma1) 
legend('true','numerical')

