%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code use a tracking model to varify Propositions 4-6 of Miao, Wu and Young (2020)
% target y_t = G*x_t, state x_t= rho*x_{t-1}+ epsilon_t; Objective is mean squared error
%
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by Jianjun Miao, Department of Economics, Boston University
% Emai: miaoj@bu.edu
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all; clear all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameter values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

beta = 1;  %Subjective discount factor
lambda = 2; %Information cost

n = 4; % State dimension 
nu = 4; % Control dimension 

rho = 0.9; %1; 2; % Persistence
A = rho*eye(n,n);  %State transition matrix

W = eye(n,n); % Innovation variance 
W(1,1) = 0.9; 
W(2,2) = 0.8;
W(3,3) = 0.7;
W(4,4) = 0.5;

G = ones(nu,n); % control z = G*\hat{x_t} for tracking problem
%Choose any numbers of interest to change the rank of Omega, which affects
%the signal dimension
G(1,1) = 2; % set to 1 when rGho>=1 to generate an example with no solution. 
G(2,2) = 3;
G(3,3) = 4;
% G = ones(1,n); % nu =1;
Omega = G'*G; % One period benefit matrix of information

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Call our RI steady state solver, work for any rho
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Sigma_pr0 = A*A'+ W;
Theta0 = Omega;
[Sigma, Sigma_pr, Theta, Lambda] = RI_SS_FOC(lambda,beta,A,W, Omega, Theta0, Sigma_pr0);
disp('Our numerical solution')
Sigma

disp('Eigenvalue of Omega')   
    eig(Omega)  
if abs(rho) >= 1 
    disp('All states are nonstationary. The benefit matrix Omega must be invertible.')
    if min(abs(eig(Omega)))< 1e-8
    disp('Omega is not invertible. No solution')
    return
    end
%Omega = eye(n) % \Omega must be invertible
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% When |rho| < 1, and Omega is rank one, check Proposition 6 in MWY
% (2020)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if abs(rho) <1 & rank(Omega) ==1 
sigmahat1=0.5/rho^2*(sqrt(1+2*rho^2*lambda/norm(sqrtm(W)*G')^2)-1);
disp('True solution by our proposition when |rho|< by Proposition 6')
Sigma_true = W/(1-rho^2)-W*Omega*W/(norm(sqrtm(W)*G'))^2*((1-rho^2)^(-1)-sigmahat1)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check Propositions 4 and 5 for stationary process
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if abs(rho)<1
    
    [U,D,v] = svd(sqrtm(W)*Omega*sqrtm(W)) ;

    for i=1:n;
        h(i)= 1/2/rho^2*(sqrt(1+2*rho^2*lambda/D(i,i))-1);
    end
    Sigma_hat= diag(h);

     % theoretical solution
    disp('True solution when |rho|< 1 from Proposition 4')
    Sigma = sqrtm(W)*U*min(Sigma_hat,1/(1-rho^2))*U'*sqrtm(W)
    InvSqrtW = inv(sqrtm(W));
    MaxM = max(0,2*D/lambda.*(eye(n)-(1-rho^2)*Sigma_hat));
    SNR = InvSqrtW*U*MaxM*U'*InvSqrtW; %C'*V^(-1)*C = SNR;
      
    r = 0;
     for i=1:n;
        if 2*D(i,i)/(1-rho^2)>lambda
        r= r+1;
        end
     end
    disp(['True signal dimension by Proposition 5: ' num2str(r)])

    disp('Numerical results for signal form')
    Sigma_pr =A*Sigma*A'+W; 
    [C,V,r] = RI_SIG1(Sigma,Sigma_pr)
    
    disp('Check if numerical and true SNRs are the same')
    norm(C'*inv(V)*C - SNR, 'fro')/norm(SNR,'fro')

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% When abs(rho)>=1, and Omega is invertible, check Proposition 4
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if abs(rho) >= 1 & abs(eig(Omega))>1e-10
    disp('Omega is invertible')
    [U,D,v] = svd(sqrtm(W)*Omega*sqrtm(W));
    for i=1:n;
    h(i)= 1/2/rho^2*(sqrt(1+2*rho^2*lambda/D(i,i))-1);
    end
    Sigma_hat= diag(h);

 % theoretical solution
    disp('True solution when |rho| >=1 from Propostion 4')
    Sigma = sqrtm(W)*U*Sigma_hat*U'*sqrtm(W) 
    InvSqrtW = inv(sqrtm(W));
    MaxM = max(0,2*D/lambda.*(eye(n)-(1-rho^2)*Sigma_hat));
    SNR = InvSqrtW*U*MaxM*U'*InvSqrtW; %C'*V^(-1)*C = SNR;
 
    disp('Signal dimension from numerical solution')
    Sigma_pr =A*Sigma*A'+W; 
    [C,V,r] = RI_SIG1(Sigma,Sigma_pr)
    disp('Check if numerical and true SNRs are the same')
    norm(C'*inv(V)*C - SNR, 'fro')
    
end


%Afrouzi and Yang (2020) solution

omega = 0.5*lambda; %their omega corresponds to our lamba/2
Q =sqrtm(W); 
H = G';
Sigma0 = W;
[Sigma1,Sigma_p,Lambda] = Solve_RI_Dynamics(omega,beta,A,Q,H,Omega,Sigma0);                
disp('Afrouzi and Yang solution')
Sigma_p




